#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>
#include <rtl.h>
#include <rtl_fifo.h>
#include <pthread.h>

#include "events.h"
#include "periodServer.h"
#include "Kalman.h"
#include "shared.h"

#include "debug.h"
#define DEBUG_MODULE_NAME KALMAN

#define FIFO 1

/***************** Variables globales  *******************/

 periodicTask* LeftLineSensor;
 Event* LeftLineEvent;

 periodicTask* RightLineSensor;
 Event* RightLineEvent;
 
 Event* LineEvent; //( ou de gauche ou de droite )

 //P-e un mutex sur la fifo pour quand odometre
 
 pthread_t LineDetectorThreadId;
 
int fifo;    //la fifo

double memodo1;
double memodo2;


/************ Initialisation du serveur de detection de ligne blanche **********/
int write_fifo (double d)
{
  rtf_put(FIFO, (char*) &d,sizeof(double));
}


lineServer_init() 
{
  RightLineSensor  = createPeriodicAcquisition(I2C_LEFT_LINE_SENSOR_SLAVE, 0 , 1 << I2C_LEFT_LINE_SENSOR_BIT, 5000);
  LeftLineSensor  = createPeriodicAcquisition(I2C_RIGHT_LINE_SENSOR_SLAVE, 0 , 1 << I2C_RIGHT_LINE_SENSOR_BIT, 5000);

  RightLineEvent = i2c_getEvent(LeftLineSensor);
  LeftLineEvent = i2c_getEvent(RightLineSensor);
	
  if (!rtf_create (FIFO, 10000))
   {
	debug_printf(DEBUG_MODULE_NAME,"Probleme avec la FIFO \n"); 
        exit (0);	
   }

  // Initialisation du canal ( Positionnement ) 
  write_fifo(1);
  write_fifo(-200);
  write_fifo(1050);
  write_fifo(0);
  write_fifo(10);
  write_fifo(0.1);
    
 
  pthread_create(&LineDetectorThreadId,NULL,LineDetectorThread,NULL); 

}



void * LineDetectorThread (void * arg)
{
   while (1)
   {
     orEvents(&LeftLineEvent, &RightLineEvent, &Line);
     waitEvent(&Line);

     if (Line.signalingEvent == &LeftLineEvent)
      {
 write_fifo(3);
 write_fifo(2);
      
      }
     else
      {
 write_fifo(3);
 write_fifo(1);
      
      }
     
   }
}

void * OdometreThread (void * arg)
{
  double odo1,odo2;

  //ici, on ajoute un bout de machin qui fait que odo1 et odo2 
  //representent maintenant l'ecart entre la nouvelle position des 
  //points de contact des roues et la position qu'ils avaient 
  //lors du dernier appel de cette fonction


  odo1=globalState->rightLength-memodo1;
  odo2=globalState->leftLength-memodo2;

  memodo1=globalState->rightLength;
  memodo2=globalState->leftLength;

 
  while(1){

  if(odo1 > -100 && odo2 > -100){

    write_fifo(2);
    write_fifo(odo1);
    write_fifo(odo2);
    usleep(10000); //tous les 1/100 secondes

  }

  }


}


int readFifo(){
  unsigned i;
  read (fifo,&i,sizeof(int));
  //remarque, on ne gere pas le cas ou  le processus RT n'a pas 
  //fini d'ecrire dans la fifo, parceque c'est impossible

 return i;
}


//------------------------------------------------------------------
//   methode pour ouvrir une fifo
//------------------------------------------------------------------

void openFIFO(){
  fifo=0;
  while(fifo<0){
    fifo=open("/dev/rtf1",O_RDONLY);
    usleep(10000);
  }

}


void * ThreadDeCalcul (void *arg) 
{
  double d;
  void * buff;
  int size;

  openFIFO();

  while(1){
    size=sizeof(double);
    buff=(void *)&d;
    
    while(size>0){
      size-=read(fifo,buff,size);
      if(size<0) return; //normalement a arrive jamais sur une fifo
      if(size<sizeof(double)) buff=(void*)&d+sizeof(double)-size;
      
    }

    interprete_kalman(d);
  }


}





/***************** Lancement et Arret *******************/

void Kalman_initModule()
{
  debug_printf(DEBUG_MODULE_NAME,"Initializing Kalman \n");
  lineServer_init();
  // odometre_init()    A completer
}

void Kalman_cleanUpModule()
{
  
  deletePeriodicAcquisition(LeftLineSensor);
  deletePeriodicAcquisition(RightLineSensor);
	
  pthread_cancel (&LineDetectorTreadId);
  
  debug_printf(DEBUG_MODULE_NAME,"Cleaning Kalman\n");
}













