#include <gtk/gtk.h>  
#include <pthread.h>
#include <string.h>
#include <errno.h>

#include <stdio.h>

#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "comManager.h"

#include "gtkMutex.h"

#include "mainWindow.h"

static int isConnected = 0;

static GtkWidget *disconnectButton, *connectButton;

char* robot_host;
int robot_port;
int socketfd;


pthread_mutex_t mutexRobotConnection;


pthread_t connectThread;
pthread_t pingThread;
pthread_t readThread;

typedef struct NetCallBack {
  char* prefix;
  void (*callback)(char*);
  struct NetCallBack *next;
} NetCallBack;

NetCallBack* netCallBackList = NULL;

void registerNetCallBack(char* prefix, void (*callback)(char*))
{
  NetCallBack* netCallBackElem;

  netCallBackElem = (NetCallBack*) malloc(sizeof (NetCallBack));
  netCallBackElem->prefix = strdup(prefix);
  netCallBackElem->callback=callback;
  netCallBackElem->next=NULL;
  
  if (netCallBackList == NULL)
    netCallBackList = netCallBackElem;
  else
    {
      netCallBackElem->next = netCallBackList;
      netCallBackList = netCallBackElem;
    }

  printf ("register NetCallBack 0x%x (next : 0x%x - str : %s)\n",netCallBackElem,netCallBackElem->next,netCallBackElem->prefix);
}

static pongReceived = 1;

void pongCallBack()
{
	printf("Pong callback\n");
	pongReceived = 1;
        mainWindow_addLog(">PONG");
}

void pingThreadBody(void * junk)
{
  char buffer[200];
  int dummy;
  char c;

  pthread_setcancelstate(PTHREAD_CANCEL_ENABLE,NULL);
  pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS,NULL);


  registerNetCallBack("PONG",pongCallBack);
  
  while (1)
    {
      if (!pongReceived)
      {
	      printf("PONG not received !!!!\n");
      }
      else
	      pongReceived=0;
      
      mainWindow_addLog("<PING");
      

      printf("Send ping...\n");
      strcpy(buffer,"PING\n");
      
      pthread_mutex_lock(&mutexRobotConnection);
//      write(socketfd, buffer, strlen(buffer));
      if (write(socketfd, buffer, strlen(buffer)) < 0)
	{
	  printf("Error ping...\n");
	  //pthread_mutex_unlock(&mutexRobotConnection);	  
	  //sprintf(buffer,"Unable to send the ping request (%s)...",strerror(errno));
	  //mainWindow_addLog(buffer);
	  //pthread_mutex_lock(&mutexGTKLoop);
	  //gtk_widget_set_sensitive(connectButton,TRUE);
	  //gtk_widget_set_sensitive(disconnectButton,FALSE);
	  //pthread_mutex_unlock(&mutexGTKLoop);
	  //return;
	}

      pthread_mutex_unlock(&mutexRobotConnection);

      printf("Waiting pong...\n");

//      dummy=0;
//      while ((dummy != -1) && (c != '\n'))
//	{
//	  dummy = read(socketfd,&buffer,1);
//	  if (dummy > 0)
//	    printf("%c\n",buffer[0]);
//	}

            sleep(2);
    }
}

void readThreadBody()
{
  int dummy;
  char buffer[100];
  char c;
  int index;
  char* substring;
  NetCallBack* p;

  while (1)
    {
      printf("READ...\n");
      index = 0;
      c='\0';
      while ((dummy >= 0) && (c != '\n'))
	{
	  dummy = read(socketfd,&c,1);
	  if (dummy > 0)
	    buffer[index++]=c;
	}
      buffer[index]='\0';

      if (c != '\n')
	{
	  printf("Exiting reader thread (%s)\n",strerror(errno));
	  return;
	}
      else
	{
	  printf(">> %s\n",buffer);
	  p = netCallBackList;
	  while (p != NULL)
	  {
	    substring = strstr(buffer,p->prefix);
	    if (substring != NULL)
	      if (strlen(substring)==strlen(buffer))
		{
		  printf("handler found...\n");
		  p->callback(buffer);
		}
	    p=p->next;
	    
	  }
	}
    }
}

void connectThreadBody(void * dummy)
{
  char buffer[200];
  struct sockaddr_in robot_addr;

  pthread_mutex_lock(&mutexRobotConnection);

  sprintf(buffer,"Contacting %s on port %d",robot_host, robot_port);
  mainWindow_addLog(buffer);

  if ((socketfd = socket(AF_INET,SOCK_STREAM, 0))<0)
    {
      sprintf(buffer,"Unable to create the socket (%s)...",strerror(errno));
      mainWindow_addLog(buffer);
      gtk_widget_set_sensitive(connectButton,TRUE);
      gtk_widget_set_sensitive(disconnectButton,FALSE);
      return;
    }

  robot_addr.sin_family=AF_INET;
  robot_addr.sin_port=htons(robot_port);
  robot_addr.sin_addr.s_addr=inet_addr(robot_host);
  bzero(&(robot_addr.sin_zero),8);

  if (( connect(socketfd, 
		 (struct sockaddr*)&robot_addr,
		 sizeof(struct sockaddr)))<0)
    {
      sprintf(buffer,"Unable to connect to the robot (%s)...",strerror(errno));
      mainWindow_addLog(buffer);
      gtk_widget_set_sensitive(connectButton,TRUE);
      gtk_widget_set_sensitive(disconnectButton,FALSE);
      return;
    }

  mainWindow_addLog("Connected to the robot");
  isConnected=1;

  pthread_mutex_unlock(&mutexRobotConnection);

  pthread_create(&pingThread, NULL, &pingThreadBody, NULL);
  pthread_create(&readThread, NULL, &readThreadBody, NULL);
}

static void robot_connect (GtkWidget *widget, gpointer data)
{
  mainWindow_addLog_NM("Trying to connect to robot...");
  gtk_widget_set_sensitive(disconnectButton,TRUE);
  gtk_widget_set_sensitive(connectButton,FALSE);
  pthread_create(&connectThread, NULL, &connectThreadBody, NULL);
}


static void robot_disconnect (GtkWidget *widget, gpointer data)
{
  mainWindow_addLog_NM("Disconnect from the robot...");
  gtk_widget_set_sensitive(connectButton,TRUE);
  gtk_widget_set_sensitive(disconnectButton,FALSE);
  pthread_cancel(&pingThread);
}



GtkWidget *createConnectFrame()
{
    GtkWidget *frame;
  GtkWidget *vbox;
  
  frame = gtk_frame_new(" Connection ");
  gtk_widget_show (frame);

  vbox = gtk_vbox_new (FALSE, 0);
  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show (vbox);

  gtk_container_add (GTK_CONTAINER (frame), vbox);

  connectButton = gtk_button_new_with_label ("Connect");

  gtk_signal_connect (GTK_OBJECT (connectButton), "clicked",
                        GTK_SIGNAL_FUNC (robot_connect), NULL);

  if (isConnected)    
    gtk_widget_set_sensitive(connectButton,FALSE);

  gtk_widget_show (connectButton);
  gtk_box_pack_start (GTK_BOX (vbox), connectButton, FALSE, FALSE, 5);


  disconnectButton = gtk_button_new_with_label ("Disconnect");

  gtk_signal_connect (GTK_OBJECT (disconnectButton), "clicked",
		      GTK_SIGNAL_FUNC (robot_disconnect), NULL);


  if (!isConnected)    
    gtk_widget_set_sensitive(disconnectButton,FALSE);

  gtk_widget_show (disconnectButton);
  gtk_box_pack_start (GTK_BOX (vbox), disconnectButton, FALSE, FALSE, 5);

  
  return frame;
}


static initPage()
{
  GtkWidget *empty_notebook_page;
  GtkWidget *label;
  GtkWidget *vbox;
  
  vbox = gtk_vbox_new (FALSE, 0);
  empty_notebook_page = vbox;
  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show (empty_notebook_page);  

  
  gtk_box_pack_start (GTK_BOX (vbox), createConnectFrame(), FALSE, FALSE, 5);
  

  mainWindow_addNoteBookPage(empty_notebook_page);
  label = gtk_label_new ("Ethernet");
  mainWindow_setNoteBookPageName(empty_notebook_page, label);

}

void comManager_init()
{
  initPage();

  pthread_mutex_init(&mutexRobotConnection,NULL);

  mainWindow_addLog("Communication module initialized...");
}

void comManager_setPort(int port)
{
  robot_port=port;
}

void comManager_setHost(char* host)
{
  robot_host=strdup(host);
}
