#include "drawingArea.h"
#include <stdlib.h>
#include <xmlmemory.h>
#include <parser.h>
#include <math.h>
static GtkWidget *drawingArea;

#define pi 3.14159

Layer* layers = NULL;

static GdkPixmap *pixmaps[NB_OF_LAYERS];
static GdkPixmap *masks[NB_OF_LAYERS];

int pixmapInitialized =0;

static xmlNodePtr xmlDrawNodePtr = NULL;
static xmlDocPtr xmlDrawDocPtr = NULL;

static float ZOOM_FACTOR = 1.0/10.0;

static int xMax;
static int yMax;

inline static
int real2draw(int realCoord)
{
  return (int)((double)realCoord*ZOOM_FACTOR);
}

/** handling client calls **/
Layer* addDrawingLayer(void (*createBitmapProc)(struct Layer*))
{
  Layer* newLayer;

  newLayer = (Layer*) malloc(sizeof(Layer));
  

  newLayer->createBitmapProc = createBitmapProc;
  
  newLayer->isVisible = 1;
  newLayer->rotX = 0.0;
  newLayer->rotY = 0.0;
  newLayer->rotTheta = 0.0;
  
  if (layers)
    {
      newLayer->next=layers;
      layers=newLayer;
    }
  else
    {
      newLayer->next=NULL;
      layers=newLayer;
    }
  
  printf("0x%x - 0x%x -%lf - %lf\n",newLayer->pixmap,newLayer->mask,xMax,yMax);

  return newLayer;
}

void setRotation(Layer* layer, double x, double y, double theta)
{
  layer->rotX = x;
  layer->rotY = y;
  layer->rotTheta = theta;
}

void scalePoint(GdkPoint p, GdkPoint* r)
{
  r->x = (int)((double)p.x*ZOOM_FACTOR);
  r->y = (int)((double)p.y*ZOOM_FACTOR);
}

void rotatePoint(GdkPoint ref, double theta, GdkPoint p, GdkPoint *result)
{
  GdkPoint pr;

  pr.x = p.x - ref.x;
  pr.y = p.y - ref.y;
  
  result->x = ref.x + pr.x * cos(theta) - pr.y * sin(theta);
  result->y = ref.y + pr.y * cos(theta) + pr.x * sin(theta);
}




void setZoom(float zoom)
{
  ZOOM_FACTOR=zoom;
  printf("setZoom : %f\n",zoom);
  deleteBitmap();
  createBitmap(drawingArea);
  gtk_widget_queue_draw(drawingArea);
}



/* Cration d'un nouveau pixmap d'arrire-plan de la taille voulue */
static gint
configure_event (GtkWidget *widget, GdkEventConfigure *event)
{  
  printf("drawingArea : configure_event\n");
  

  return TRUE;
}

void draw_rectangle(int layer, GtkWidget *widget, GdkGC* gc, int fill, int x, int y, int w, int h)
{

  gdk_draw_rectangle (pixmaps[layer],
		      gc,
		      TRUE,
		      real2draw(x), real2draw(y),
		      real2draw(w), real2draw(h));
  
  gdk_draw_rectangle (masks[layer],
		      widget->style->white_gc,
		      TRUE,
		      real2draw(x), real2draw(y),
		      real2draw(w), real2draw(h));

}

void drawPoly(Layer* layer, char* colorStr, int fill, GdkPoint *points, gint npoints)
{ 
  GdkGC* gc;
  GdkColor color;
  int i;
  GdkPoint dummyPoint;
  GdkPoint refPoint;

  gc=gdk_gc_new(drawingArea->window);
  gdk_gc_copy(gc,drawingArea->style->white_gc);
  
  gdk_color_parse(colorStr,&color);
  gdk_colormap_alloc_color(gtk_widget_get_colormap(drawingArea), &color, FALSE, TRUE);

  gdk_gc_set_foreground(gc,&color);

  printf("0x%x - 0x%x\n",layer->pixmap,layer->mask);

  refPoint.x = layer->rotX;
  refPoint.y = layer->rotY;

  for (i=0 ; i < npoints ; i++)
    {
      rotatePoint(refPoint, layer->rotTheta, points[i], &dummyPoint);
      points[i]=dummyPoint;
      scalePoint(points[i], &dummyPoint);
      points[i]=dummyPoint;      
    }


  gdk_draw_polygon    (layer->pixmap,
                       gc,
                       fill,
                       points,
                       npoints);

  gdk_draw_polygon    (layer->mask,
                       drawingArea->style->white_gc,
                       fill,
                       points,
                       npoints);

  
#if 0
  gdk_draw_rectangle (layer->pixmap,
		      gc,
		      TRUE,
		      real2draw(x), 
		      real2draw(y),
		      real2draw(w), real2draw(h));
  
  gdk_draw_rectangle (layer->mask,
		      drawingArea->style->white_gc,
		      TRUE,
		      real2draw(x), real2draw(y),
		      real2draw(w), real2draw(h));
#endif

}

void drawLine(Layer* layer, char* colorStr, double x1, double y1, double x2, double y2)
{ 
  GdkGC* gc;
  GdkColor color;
  GdkPoint p1,p2,refPoint,dummyPoint;

  gc=gdk_gc_new(drawingArea->window);
  gdk_gc_copy(gc,drawingArea->style->white_gc);
  
  gdk_color_parse(colorStr,&color);
  gdk_colormap_alloc_color(gtk_widget_get_colormap(drawingArea), &color, FALSE, TRUE);

  gdk_gc_set_foreground(gc,&color);

  p1.x = x1;
  p1.y = y1;
  p2.x = x2;
  p2.y = y2;

  refPoint.x = layer->rotX;
  refPoint.y = layer->rotY;

  rotatePoint(refPoint, layer->rotTheta, p1, &dummyPoint);
  p1 = dummyPoint;
  scalePoint(p1, &dummyPoint);
  p1 = dummyPoint;
  rotatePoint(refPoint, layer->rotTheta, p2, &dummyPoint);
  p2 = dummyPoint;
  scalePoint(p2, &dummyPoint);
  p2 = dummyPoint;

  gdk_draw_line    (layer->pixmap,
		    gc,
		    p1.x, p1.y,
		    p2.x, p2.y);
  gdk_draw_line    (layer->mask,
		    drawingArea->style->white_gc,
		    p1.x, p1.y,
		    p2.x, p2.y);


}

void drawRectangle(Layer* layer, char* colorStr, int fill, int x, int y, int w, int h)
{ 
  GdkPoint p[4];

  p[0].x = x;
  p[0].y = y;
  p[1].x = x+w;
  p[1].y = y;
  p[2].x = x+w;
  p[2].y = y+h;
  p[3].x = x;
  p[3].y = y+h;

  drawPoly(layer, colorStr, fill, p, 4);
}

void draw_circle(int layer, GtkWidget *widget, GdkGC* gc, int fill, int x, int y, int r)
{
  
  gdk_draw_arc (pixmaps[layer],
		gc,
		TRUE,
		real2draw(x-(r/2)), real2draw(y-(r/2)),		
		real2draw(r), real2draw(r),
		0, 360*64);

  gdk_draw_arc (masks[layer],
		widget->style->white_gc,
		TRUE,
		real2draw(x-(r/2)), real2draw(y-(r/2)),		
		real2draw(r), real2draw(r),
		0, 360*64);
  
}

void handleRect(xmlNodePtr cur, GtkWidget *widget, GdkPixmap *pixmap )
{
  unsigned char xStr[50];
  unsigned char yStr[50];  
  unsigned char hStr[50];
  unsigned char wStr[50];
  unsigned char colorStr[50];
  int x,y,h,w;
    
  GdkGC* gc;
  GdkColor color;

  sprintf(xStr,"%s",xmlGetProp(cur, "x"));
  x=atoi(xStr);
  sprintf(yStr,"%s",xmlGetProp(cur, "y"));
  y=atoi(yStr);
  sprintf(hStr,"%s",xmlGetProp(cur, "h"));
  h=atoi(hStr);
  sprintf(wStr,"%s",xmlGetProp(cur, "w"));
  w=atoi(wStr);
  sprintf(colorStr,"%s",xmlGetProp(cur, "color"));

  printf("Handling rect (x : %d - y : %d - h : %d - w : %d)\n",x,y,h,w);

  gc=gdk_gc_new(widget->window);
  gdk_gc_copy(gc,widget->style->white_gc);

  gdk_color_parse(colorStr,&color);
  gdk_colormap_alloc_color(gtk_widget_get_colormap(widget), &color, FALSE, TRUE);

  gdk_gc_set_foreground(gc,&color);
//  gdk_gc_set_background(gc,color);
  
  draw_rectangle (BACKGROUND_LAYER,
		  widget,
		  gc,
		  TRUE,
		  x, y,
		  w, h);

  printf("End Handling rect\n");

}


void handleCircle(xmlNodePtr cur, GtkWidget *widget, int layer )
{
  unsigned char xStr[50];
  unsigned char yStr[50];  
  unsigned char rStr[50];
  unsigned char colorStr[50];
  int x,y,r;
    
  GdkGC* gc;
  GdkColor color;

  sprintf(xStr,"%s",xmlGetProp(cur, "x"));
  x=atoi(xStr);
  sprintf(yStr,"%s",xmlGetProp(cur, "y"));
  y=atoi(yStr);
  sprintf(rStr,"%s",xmlGetProp(cur, "r"));
  r=atoi(rStr);
  sprintf(colorStr,"%s",xmlGetProp(cur, "color"));

  printf("Handling circle (x : %d - y : %d - r : %d)\n",x,y,r);

  gc=gdk_gc_new(widget->window);
  gdk_gc_copy(gc,widget->style->white_gc);


  gdk_color_parse(colorStr,&color);
  gdk_colormap_alloc_color(gtk_widget_get_colormap(widget), &color, FALSE, TRUE);

  gdk_gc_set_foreground(gc,&color);
//  gdk_gc_set_background(gc,color);
  
  draw_circle (BACKGROUND_LAYER,
	       widget,
	       gc,
	       TRUE,
	       x, y,
	       r);
}

#if 0
void drawRobot(int layer, double x, double y, double theta)
{
  GdkGC* gc;
  GdkColor color;
  GtkWidget* widget = drawingArea;
  
  gc=gdk_gc_new(widget->window);
  gdk_gc_copy(gc,widget->style->white_gc);
  
  
  gdk_color_parse("yellow",&color);
  gdk_colormap_alloc_color(gtk_widget_get_colormap(widget), &color, FALSE, TRUE);
  
  gdk_gc_set_foreground(gc,&color);
  
  draw_circle (ROBOT_LAYER,
	       widget,
	       gc,
	       TRUE,
	       x, y,
	       400);
  
  gtk_widget_queue_draw(widget);
}

#endif

void eraseLayer(Layer* layer)
{
  gdk_draw_rectangle (layer->pixmap,
		      drawingArea->style->white_gc,
		      TRUE,
		      real2draw(0), real2draw(0),
		      real2draw(layer->xMax), real2draw(layer->yMax));
  
  gdk_draw_rectangle (layer->mask,
		      drawingArea->style->black_gc,
		      TRUE,
		      real2draw(0), real2draw(0),
		      real2draw(layer->xMax), real2draw(layer->yMax));
}

void drawingAreaRedraw()
{
  gtk_widget_queue_draw(drawingArea);
}



#define ROBOT_RADIUS 200.0
#define ROBOT_L1 40.0
#define ROBOT_L2 50.0
#define ROBOT_L3 150.0
#define ROBOT_COLOR "yellow"
void drawRobot(Layer* layer, int fill, double x, double y, double theta)
{
  printf("Drawing robot at %lf,%lf,%f on layer 0x%x\n",x,y,theta,layer);

  setRotation(layer, 1250.0, 1250.0, theta);
  drawRectangle(layer,ROBOT_COLOR, fill, x-ROBOT_RADIUS, y-ROBOT_L1, ROBOT_RADIUS + ROBOT_L3, 2*ROBOT_L1);
  drawRectangle(layer,ROBOT_COLOR, fill, x-ROBOT_L2, y-ROBOT_RADIUS, 2*ROBOT_L2, 2*ROBOT_RADIUS);
  drawLine(layer,ROBOT_COLOR,x-ROBOT_L2, y-ROBOT_L1, x+ROBOT_L2, y+ROBOT_L1);
  drawLine(layer,ROBOT_COLOR,x-ROBOT_L2, y+ROBOT_L1, x+ROBOT_L2, y-ROBOT_L1);
  setRotation(layer, 1250.0, 1250.0, 0.0);
}


void deleteBitmap()
{
/*
  int i;

  for (i=0 ; i<NB_OF_LAYERS ; i++)
    {
      gtk_object_unref (GTK_OBJECT(pixmaps[i]));
      gtk_object_unref (GTK_OBJECT(masks[i]));
    }
*/
}

void createBitmap(GtkWidget *widget)
{
  xmlNodePtr cur;

  unsigned char xMaxStr[50];
  unsigned char yMaxStr[50];

  Layer* currentLayer;

  int i;
      
  printf("Creating bitmap (widget : 0x%x)...\n",(int)widget);

  printf("drawingArea : doc : 0x%x - node : 0x%x\n",(int)xmlDrawDocPtr,(int)xmlDrawNodePtr);
  sprintf(xMaxStr,"%s",xmlGetProp(xmlDrawNodePtr, "xMax"));
  sprintf(yMaxStr,"%s",xmlGetProp(xmlDrawNodePtr, "yMax"));
  
  xMax = atoi(xMaxStr);
  yMax = atoi(yMaxStr);

  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingArea), real2draw(xMax), real2draw(yMax));
  
  // xmlNodeListGetString(xmlDrawDocPtr, xmlDrawNodePtr, 1);
  
  for (i=0 ; i<NB_OF_LAYERS ; i++)
    {
      pixmaps[i] = gdk_pixmap_new(widget->window,
				  real2draw(xMax),
				  real2draw(yMax),
				  -1);

      gdk_draw_rectangle (pixmaps[i],
			  widget->style->white_gc,
			  TRUE,
			  real2draw(0), real2draw(0),
			  real2draw(xMax), real2draw(yMax));
      
      masks[i] = gdk_pixmap_new(widget->window,
				real2draw(xMax),
				real2draw(yMax),
				-1);

      gdk_draw_rectangle (masks[i],
			  widget->style->black_gc,
			  TRUE,
			  real2draw(0), real2draw(0),
			  real2draw(xMax), real2draw(yMax));

    }


  cur = xmlDrawNodePtr->xmlChildrenNode;
  while (cur != NULL)
    {
      printf("%s\n",cur->name);
      
      if (!strcmp(cur->name,"rect"))
	handleRect(cur,widget,BACKGROUND_LAYER);

      if (!strcmp(cur->name,"circle"))
	handleCircle(cur,widget,BACKGROUND_LAYER);

      cur = cur->next;
    }

/*
  gdk_draw_rectangle (pixmap,
		      widget->style->white_gc,
		      TRUE,
			  0, 0,
		      widget->allocation.width,
		      widget->allocation.height);

      

  draw_rectangle (ROBOT_LAYER,
		  widget,
		  widget->style->white_gc,
		  TRUE,
		  50,50,
		  widget->allocation.width-100,
		  widget->allocation.height-100);

*/

  currentLayer=layers;
  while (currentLayer != NULL)
    {
      currentLayer->pixmap = gdk_pixmap_new(drawingArea->window,
					real2draw(xMax),
					real2draw(yMax),
					-1);
      currentLayer->xMax=xMax;
      currentLayer->yMax=yMax;
      gdk_draw_rectangle (currentLayer->pixmap,
			  widget->style->white_gc,
			  TRUE,
			  real2draw(0), real2draw(0),
			  real2draw(xMax), real2draw(yMax));
     
      currentLayer->mask = gdk_pixmap_new(drawingArea->window,
					  real2draw(xMax),
					  real2draw(yMax),
					  -1);
      gdk_draw_rectangle (currentLayer->mask,
			  widget->style->black_gc,
			  TRUE,
			  real2draw(0), real2draw(0),
			  real2draw(xMax), real2draw(yMax));

      currentLayer->createBitmapProc(currentLayer);
      currentLayer=currentLayer->next;
    }


//  drawRobot(ROBOT_LAYER,
//	    100.0,
//	    100.0,
//	    0.0);
	    
    
}



/* Remplit l'cran  partir du pixmap d'arrire-plan */
static gint
expose_event (GtkWidget *widget, GdkEventExpose *event)
{  
  GdkGC* gc;
  int i;
  Layer* currentLayer;

  printf("drawingArea : expose_event\n");
  if (!pixmapInitialized)
    {
      createBitmap(widget);
      pixmapInitialized=1;
    }

  gc=gdk_gc_new(widget->window);
  gdk_gc_copy(gc,widget->style->white_gc);

  gdk_draw_pixmap(widget->window,
		  gc,
		  pixmaps[0],
		  event->area.x, event->area.y,
		  event->area.x, event->area.y,
		  event->area.width, event->area.height);
  
  for (i=1 ; i<NB_OF_LAYERS ; i++)
    {
      gdk_gc_set_function(gc,GDK_OR);
      gdk_draw_pixmap(widget->window,
		      gc,
		      masks[1],
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);
      
      
      gdk_gc_set_function(gc,GDK_AND);
      gdk_draw_pixmap(widget->window,
		      gc,
		      pixmaps[1],
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);
    }

  currentLayer=layers;
  while (currentLayer != NULL)
    {
      gdk_gc_set_function(gc,GDK_OR);
      gdk_draw_pixmap(widget->window,
		      gc,
		      currentLayer->mask,
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);
      
      
      gdk_gc_set_function(gc,GDK_AND);
      gdk_draw_pixmap(widget->window,
		      gc,
		      currentLayer->pixmap,
		      event->area.x, event->area.y,
		      event->area.x, event->area.y,
		      event->area.width, event->area.height);

      currentLayer=currentLayer->next;
    }


  return FALSE;
}

void setXMLBackground(xmlDocPtr doc, xmlNodePtr cur)
{
  printf("drawingArea : setXMLBackground\n");
  xmlDrawNodePtr = cur;
  xmlDrawDocPtr = doc;  
}

GtkWidget *drawingArea_init()
{
  drawingArea = gtk_drawing_area_new();
  gtk_drawing_area_size(GTK_DRAWING_AREA(drawingArea), 200, 200);

  gtk_signal_connect (GTK_OBJECT (drawingArea), "expose_event",
                      (GtkSignalFunc) expose_event, NULL);
  gtk_signal_connect (GTK_OBJECT(drawingArea),"configure_event",
                      (GtkSignalFunc) configure_event, NULL);

//  gtk_signal_connect (GTK_OBJECT (drawing_area), "motion_notify_event",
//                      (GtkSignalFunc) motion_notify_event, NULL);
//  gtk_signal_connect (GTK_OBJECT (drawing_area), "button_press_event",
//                      (GtkSignalFunc) button_press_event, NULL);

  return drawingArea;
}

void drawingArea_show()
{

  gtk_widget_show (drawingArea);
}
