#include "robot.h"
#include "periodServer.h"

#include "i2cServos.h"
#include <unistd.h>

#define DEBUG_MODULE_NAME MAIN
#include "debug.h"

#include "move.h"

#define PI 3.14159

void emergencyStopCallback(Event* event)
{
  printf("Callback emergency stop...\n");

  resetStop();

  lcd_clear();
  lcd_printf(" Emergency stop\n Program halted\n");
  sleep(2);
  exit(0);
}

int main()
{
  periodicTask* jackOn;
  periodicTask* jackOff;
  Event* jackOffEvent;
  Event* jackOnEvent;

  periodicTask* frontBumperAcquisition;
  Event* frontBumperEvent;

  periodicTask* switch12VAcquisition;
  Event* switch12VEvent;

  MovementDescriptor* moveBetweenPlanet;
  Event endMoveBetweenPlanetEvent;

  MovementDescriptor* moveToBigStar;
  Event endMoveToBigStarEvent;

  MovementDescriptor* moveBack;
  Event endMoveBackEvent;

  MovementDescriptor* rotation1;
  Event endRotation1Event;

  MovementDescriptor* rotation2;
  Event endRotation2Event;

  MovementDescriptor* rotation3;
  Event endRotation3Event;

  MovementDescriptor* rotation4;
  Event endRotation4Event;

  Point *splinePoints;
  Point directionDepart;
  Point directionFin;
  MovementDescriptor* spline;
  
  Event pwmLimitationEvent;

  Event stopEvent;
  Event stopEvent2;
  
  int launchBigStarPMIOK = 1; // we are optimistic

  debug_printf(DEBUG_MODULE_NAME,"Start soft\n");

  shared_initModule();

  i2c_initModule();

  INIThctl();

  move_initModule();
  
  electro_initModule();

  relay_initModule();

  perServer_initModule();

  lcd_initModule();
  lcd_clear();

  debug_printf(DEBUG_MODULE_NAME,"BEGIN of SCENARIO\n");

  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  armMotor(OFF);

  // Gestion 12V 

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 0<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);

  lcd_clear();
  lcd_printf("Unlock Emergency\n      Stop\n");
  
  waitEvent(switch12VEvent);
  deletePeriodicAcquisition(switch12VAcquisition);

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 1<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);
  createCallback(switch12VEvent, emergencyStopCallback);
  

  // Gestion START JACK
  //******************
  jackOn  = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 1 << I2C_STARTJACK_BIT, 1 << I2C_STARTJACK_BIT, 100000);
  jackOff = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 0                     , 1 << I2C_STARTJACK_BIT, 100000);
  jackOffEvent = i2c_getEvent(jackOff);
  jackOnEvent = i2c_getEvent(jackOn);

#if 0
  lcd_clear();
  lcd_printf("\nWaiting jack off\n");
  waitEvent(jackOffEvent);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack in\n");
  waitEvent(jackOnEvent);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack off\n");
  waitEvent(jackOffEvent);

#endif

  //  lcd_clear();

  deletePeriodicAcquisition(jackOn);
  deletePeriodicAcquisition(jackOff);

  // while(1) { sleep(1); }

  // Avancement jusqu'a l'entre planettes
  moveBetweenPlanet = defineXYMovement(850.0,0.0);
  //  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveBetweenPlanet,0.9);
  setEpsilon(moveBetweenPlanet,10.0);
  setMaxSpeed(moveBetweenPlanet,30.0,30.0,MAX_SPEED_PROPORTIONAL);
  //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveBetweenPlanetEvent,"END_MOVE_BETWEEN_PLANET_EVENT");
  setEndMoveEvent(moveBetweenPlanet,  &endMoveBetweenPlanetEvent);
  
  
  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveBetweenPlanet);
  
  waitEvent(&endMoveBetweenPlanetEvent);

#if 0
  // Avancement jusqu'a la planete centrale   
  //***************************************

  moveToBigStar = defineXYMovement(1380.0,0.0);
//  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveToBigStar,0.9);
  setEpsilon(moveToBigStar,10.0);
  setMaxSpeed(moveToBigStar,30.0,30.0,MAX_SPEED_PROPORTIONAL);
      //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveToBigStarEvent,"END_MOVE_TO_BIG_STAR_EVENT");
  setEndMoveEvent(moveToBigStar,  &endMoveToBigStarEvent);

  
  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveToBigStar);

  armMotor(ON);
//  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

   waitEvent(&endMoveToBigStarEvent);

  printf("%lf - %lf - %lf\n", getX(),getY(),getTheta());

  debug_printf(DEBUG_MODULE_NAME,"End move to big star\n");

  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

  sleep(1);
  
  electroOn(1);
  
  sleep(1);
  
  armMotor(OFF);
  
      
//  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  sleep(5);

  // Recul jusqu'au point au centre des 2 planetes "chez nous"
  //**********************************************************
  moveBack = defineXYMovement(850.0,0.0);
  setRefPoint(moveBack,-5.0,0.0); //-1.0
  setPIDParams(moveBack,0.5);  //0.1
  setMaxSpeed(moveBack,20.0,20.0,MAX_SPEED_PROPORTIONAL);
  createEvent(&endMoveBackEvent,"END_MOVE_BACK_EVENT");
  setEndMoveEvent(moveBack,  &endMoveBackEvent);

  debug_printf(DEBUG_MODULE_NAME,"Start move back\n");
  startMove(moveBack);
  waitEvent(&endMoveBackEvent);

#endif // 0

  sleep(5);

  setServoPos(LEFT_SERVO,OUT_LEFT_SERVO);
  
  // Rotation et depose du premier drapeau
  //**************************************
  rotation1 = defineRotationMovement(PI/2);
  setMaxSpeed(rotation1,0,30.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation1Event,"END_ROTATION_1_EVENT");
  setEndMoveEvent(rotation1,  &endRotation1Event);
  startMove(rotation1);
  waitEvent(&endRotation1Event);
  electroOn(2);
  sleep(1);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);


  // Retour au centre
  //******************
  rotation2 = defineRotationMovement(0.0);
  setMaxSpeed(rotation2,0,30.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation2Event,"END_ROTATION_2_EVENT");
  setEndMoveEvent(rotation2,  &endRotation2Event);
  startMove(rotation2);
  waitEvent(&endRotation2Event);

  setServoPos(RIGHT_SERVO,OUT_RIGHT_SERVO);

  sleep(5);

  // Rotation et depose du second drapeau
  //*************************************
  rotation3 = defineRotationMovement(-PI/2);
  setMaxSpeed(rotation3,30.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation3Event,"END_ROTATION_3_EVENT");
  setEndMoveEvent(rotation3,  &endRotation3Event);
  startMove(rotation3);  
  waitEvent(&endRotation3Event);
  electroOn(3);
  sleep(1);

  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);

  // Retour au centre
  //******************
  rotation4 = defineRotationMovement(0.0);
  setMaxSpeed(rotation4,30.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation4Event,"END_ROTATION_4_EVENT");
  setEndMoveEvent(rotation4,  &endRotation4Event);
  startMove(rotation4);  
  waitEvent(&endRotation4Event);


  sleep(5);


  // Avancement jusqu'a la planete centrale   
  //***************************************

  moveToBigStar = defineXYMovement(1400.0,0.0);
//  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveToBigStar,0.9);
  setEpsilon(moveToBigStar,10.0);
  setMaxSpeed(moveToBigStar,30.0,30.0,MAX_SPEED_PROPORTIONAL);
      //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveToBigStarEvent,"END_MOVE_TO_BIG_STAR_EVENT");
  setEndMoveEvent(moveToBigStar,  &endMoveToBigStarEvent);

  
  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveToBigStar);

  armMotor(ON);
//  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

   waitEvent(&endMoveToBigStarEvent);

  printf("%lf - %lf - %lf\n", getX(),getY(),getTheta());

  debug_printf(DEBUG_MODULE_NAME,"End move to big star\n");

  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

  sleep(2);
  
  electroOn(1);
  
  sleep(2);
  
  armMotor(OFF);
  
      
//  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  debug_printf(DEBUG_MODULE_NAME,"END OF SCENARIO...\n");

  return 0;
}




