#include "robot.h"
#include "periodServer.h"

#include "i2cServos.h"
#include <unistd.h>

#define DEBUG_MODULE_NAME MAIN
#include "debug.h"

#include "move.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <termios.h>

#include <stdio.h>

#include "motor.h"

#define BAUDRATE B9600
#define MODEMDEVICE "/dev/ttyS1"
#define _POSIX_SOURCE 1 /* POSIX compliant source */
#define FALSE 0
#define TRUE 1


#define PI 3.14159

void emergencyStopCallback(Event* event)
{
  printf("Callback emergency stop...\n");

  resetStop();

  lcd_clear();
  lcd_printf(" Emergency stop\n Program halted\n");
  sleep(2);
  exit(0);
}


volatile int STOP=FALSE;

int fd;
struct termios oldtio;
struct termios confterm;

void initSerialPort()
{
  int c, res;
  struct termios newtio;
  
  fd = open(MODEMDEVICE, O_RDWR | O_NOCTTY );
  if (fd <0) {perror(MODEMDEVICE); exit(-1); }
  
  tcgetattr(fd,&oldtio); /* save current port settings */
  
  bzero(&newtio, sizeof(newtio));
  //        newtio.c_cflag = BAUDRATE | CRTSCTS | CS8 | CLOCAL | CREAD;
  newtio.c_cflag = BAUDRATE | CS8 | CLOCAL | CREAD;
  newtio.c_iflag = INPCK  | PARMRK | ISTRIP;
  newtio.c_oflag = 0;
  
  /* set input mode (non-canonical, no echo,...) */
  newtio.c_lflag = 0;
  
  newtio.c_cc[VTIME]    = 0;   /* inter-character timer unused */
  newtio.c_cc[VMIN]     = 1;   /* blocking read until 5 chars received */
  
  tcflush(fd, TCIFLUSH);
  tcsetattr(fd,TCSANOW,&newtio);
}

void initConsole()
{
  tcgetattr(0,&confterm);
  confterm.c_lflag = confterm.c_lflag & ~ICANON;
  confterm.c_cc[VMIN]=1;
  tcsetattr(0,TCSANOW,&confterm);
}

void closeSerialPort()
{
  tcsetattr(fd,TCSANOW,&oldtio);
}

#define CMD_ELECTRO_BRAS 'a'
#define CMD_SERVO_BRAS_IN 'd'
#define CMD_SERVO_BRAS_OUT 'c'
#define CMD_MOTOR_BRAS 'b'
#define CMD_PMI_RIGHT 'j'
#define CMD_PMI_LEFT 'i'
#define CMD_SERVO_RIGHT_OUT 'f'
#define CMD_SERVO_LEFT_OUT 'e'
#define CMD_SERVO_RIGHT_IN 'h'
#define CMD_SERVO_LEFT_IN 'g'
#define CMD_ELECTRO_REAR_LEFT 'o'
#define CMD_ELECTRO_REAR_RIGHT 'p'
#define CMD_ELECTRO_FRONT_LEFT 'm'
#define CMD_ELECTRO_FRONT_RIGHT 'n'
#define CMD_START_HOMOLOGATION 'q'

int interpret(unsigned char c)
{
  lcd_clear();
  lcd_setText(0,0,"");
  switch (c)
    {
    case CMD_ELECTRO_BRAS :
      lcd_printf("**  Demo mode **\n  Arm electro\n");
      onElectro(ELECTRO_ARM);
      return 0;
      break;
    case CMD_SERVO_BRAS_OUT :
      lcd_printf("**  Demo mode **\n Arm servo out \n");
      setServoPos(ARM_SERVO,MAX_ARM_SERVO);
      return 0;
      break;
    case CMD_SERVO_BRAS_IN :
      lcd_printf("**  Demo mode **\n  Arm servo in\n");
      setServoPos(ARM_SERVO,MIN_ARM_SERVO);
      return 0;
      break;
    case CMD_MOTOR_BRAS :
      lcd_printf("**  Demo mode **\n    Arm motor\n");
      //motorOn(MOTOR_2,MOTOR_SENS_1);
      armMotorOn();
      return 0;
      break;
    case CMD_PMI_RIGHT :
      lcd_printf("**  Demo mode **\n    Right PMI\n");
      onElectro(ELECTRO_RIGHT_PMI);
      return 0;
      break;
    case CMD_PMI_LEFT :
      lcd_printf("**  Demo mode **\n     Left PMI\n");
      onElectro(ELECTRO_LEFT_PMI);
      return 0;
      break;
    case CMD_SERVO_RIGHT_OUT :
      lcd_printf("**  Demo mode **\nRight servo out\n");
      setServoPos(RIGHT_SERVO,OUT_RIGHT_SERVO);
      return 0;
      break;
    case CMD_SERVO_LEFT_OUT :
      lcd_printf("**  Demo mode **\n Left servo out\n");
      setServoPos(LEFT_SERVO,OUT_LEFT_SERVO);
      return 0;
      break;
    case CMD_SERVO_RIGHT_IN :
      lcd_printf("**  Demo mode **\n Right servo in\n");
      setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);
      return 0;
      break;
    case CMD_SERVO_LEFT_IN :
      lcd_printf("**  Demo mode **\n Left servo in\n");
      setServoPos(LEFT_SERVO,IN_LEFT_SERVO);
      return 0;
      break;
    case CMD_ELECTRO_REAR_LEFT :
      lcd_printf("**  Demo mode **\nRear left electro\n");
      onElectro(ELECTRO_REAR_LEFT_FLAG);
      return 0;
      break;
    case CMD_ELECTRO_REAR_RIGHT :
      lcd_printf("**  Demo mode **\nRear right electro\n");
      onElectro(ELECTRO_REAR_RIGHT_FLAG);
      return 0;
      break;
    case CMD_ELECTRO_FRONT_LEFT :
      lcd_printf("**  Demo mode **\nFront left electro\n");
      onElectro(ELECTRO_FRONT_LEFT_FLAG);
      return 0;
      break;
    case CMD_ELECTRO_FRONT_RIGHT :
      lcd_printf("**  Demo mode **\nFront right electro\n");
      onElectro(ELECTRO_FRONT_RIGHT_FLAG);
      return 0;
      break;
    case CMD_START_HOMOLOGATION :
      lcd_printf("   Homologation\n       mode **\n\n");
      return 1;
      break;
    default :
      return 0;
    }
  return 0;
}

void console_read()
{
  unsigned char buf[255];
  int res;
  lcd_printf("**  Demo mode **\n\n");
  while (STOP==FALSE) {       /* loop for input */
    
    printf("%c -> electro bras\n",CMD_ELECTRO_BRAS);
    printf("%c -> servo bras out\n",CMD_SERVO_BRAS_OUT);
    printf("%c -> servo bras in\n",CMD_SERVO_BRAS_IN);
    printf("%c -> motor bras\n",CMD_MOTOR_BRAS);
    printf("%c -> servo right out\n",CMD_SERVO_RIGHT_OUT);
    printf("%c -> servo left out\n",CMD_SERVO_LEFT_OUT);
    printf("%c -> servo right in\n",CMD_SERVO_RIGHT_IN);
    printf("%c -> servo left in\n",CMD_SERVO_LEFT_IN);
    printf("%c -> pmi right\n",CMD_PMI_RIGHT);
    printf("%c -> pmi left\n",CMD_PMI_LEFT);
    printf("%c -> rear electro left\n",CMD_ELECTRO_REAR_LEFT);
    printf("%c -> rear electro right\n",CMD_ELECTRO_REAR_RIGHT);
    printf("%c -> front electro left\n",CMD_ELECTRO_FRONT_LEFT);
    printf("%c -> front electro right\n",CMD_ELECTRO_FRONT_RIGHT);
    printf("%c -> homologation\n",CMD_START_HOMOLOGATION);

    res = read(0,buf,1);
    printf(":%d:%c\n", buf[0],buf[0]);    
    STOP=interpret(buf[0]);
  }
}

void serial_read()
{
  unsigned char buf[255];
  int res;
  
  while (STOP==FALSE) {       /* loop for input */    
    
    printf("%c -> electro bras\n",CMD_ELECTRO_BRAS);
    printf("%c -> servo bras out\n",CMD_SERVO_BRAS_OUT);
    printf("%c -> servo bras in\n",CMD_SERVO_BRAS_IN);
    printf("%c -> motor bras\n",CMD_MOTOR_BRAS);
    printf("%c -> servo right out\n",CMD_SERVO_RIGHT_OUT);
    printf("%c -> servo left out\n",CMD_SERVO_LEFT_OUT);
    printf("%c -> servo right in\n",CMD_SERVO_RIGHT_IN);
    printf("%c -> servo left in\n",CMD_SERVO_LEFT_IN);
    printf("%c -> pmi right\n",CMD_PMI_RIGHT);
    printf("%c -> pmi left\n",CMD_PMI_LEFT);
    printf("%c -> rear electro left\n",CMD_ELECTRO_REAR_LEFT);
    printf("%c -> rear electro right\n",CMD_ELECTRO_REAR_RIGHT);
    printf("%c -> front electro left\n",CMD_ELECTRO_FRONT_LEFT);
    printf("%c -> front electro right\n",CMD_ELECTRO_FRONT_RIGHT);
    printf("%c -> homologation\n",CMD_START_HOMOLOGATION);

    res = read(fd,buf,1);
    printf(":%d:%c\n", buf[0],buf[0]);
    
    STOP=interpret(buf[0]);
  }

}


int main()
{
  periodicTask* jackOn;
  periodicTask* jackOff;
  Event* jackOffEvent;
  Event* jackOnEvent;

  periodicTask* frontBumperAcquisition;
  Event* frontBumperEvent;

  periodicTask* switch12VAcquisition;
  Event* switch12VEvent;

  MovementDescriptor* moveBetweenPlanet;
  Event endMoveBetweenPlanetEvent;

  MovementDescriptor* moveToBigStar;
  Event endMoveToBigStarEvent;

  MovementDescriptor* moveBack;
  Event endMoveBackEvent;

  MovementDescriptor* rotation1;
  Event endRotation1Event;

  MovementDescriptor* rotation2;
  Event endRotation2Event;

  MovementDescriptor* rotation3;
  Event endRotation3Event;

  MovementDescriptor* rotation4;
  Event endRotation4Event;

  Point *splinePoints;
  Point directionDepart;
  Point directionFin;
  MovementDescriptor* spline;
  
  Event pwmLimitationEvent;

  Event stopEvent;
  Event stopEvent2;
  
  int launchBigStarPMIOK = 1; // we are optimistic

  debug_printf(DEBUG_MODULE_NAME,"Start soft\n");

  shared_initModule();

  i2c_initModule();

  INIThctl();

  move_initModule();
  
  electro_initModule();

  relay_initModule();

  perServer_initModule();

  lcd_initModule();
  lcd_clear();

  debug_printf(DEBUG_MODULE_NAME,"BEGIN of SCENARIO\n");

  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  armMotor(OFF);

  initSerialPort();
//  initConsole();

//  console_read();

  serial_read();

  closeSerialPort();

  INIThctl();
  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);
	

//  motorOff(MOTOR_1); // pour etre sur...
  
  // Gestion 12V 

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 0<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);

  lcd_clear();
  lcd_printf("Unlock Emergency\n      Stop\n");
  
  waitEvent(switch12VEvent);
  deletePeriodicAcquisition(switch12VAcquisition);

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 1<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);
  createCallback(switch12VEvent, emergencyStopCallback);
  

  // Gestion START JACK
  //******************
  jackOn  = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 1 << I2C_STARTJACK_BIT, 1 << I2C_STARTJACK_BIT, 100000);
  jackOff = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 0                     , 1 << I2C_STARTJACK_BIT, 100000);
  jackOffEvent = i2c_getEvent(jackOff);
  jackOnEvent = i2c_getEvent(jackOn);

  lcd_clear();
  lcd_printf("\nWaiting jack off\n");
  waitEvent(jackOffEvent);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack in\n");
  waitEvent(jackOnEvent);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack off\n");
  waitEvent(jackOffEvent);

   lcd_clear();

  deletePeriodicAcquisition(jackOn);
  deletePeriodicAcquisition(jackOff);

  // while(1) { sleep(1); }

  // Avancement jusqu'a l'entre planettes
  moveBetweenPlanet = defineXYMovement(850.0,0.0);
  //  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveBetweenPlanet,0.9);
  setEpsilon(moveBetweenPlanet,10.0);
  setMaxSpeed(moveBetweenPlanet,30.0,30.0,MAX_SPEED_PROPORTIONAL);
  //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveBetweenPlanetEvent,"END_MOVE_BETWEEN_PLANET_EVENT");
  setEndMoveEvent(moveBetweenPlanet,  &endMoveBetweenPlanetEvent);
  
  
  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveBetweenPlanet);
  
  waitEvent(&endMoveBetweenPlanetEvent);

  sleep(5);

  
  // Rotation et depose du premier drapeau
  //**************************************
  rotation1 = defineRotationMovement(PI/2);
  setMaxSpeed(rotation1,0,30.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation1Event,"END_ROTATION_1_EVENT");
  setEndMoveEvent(rotation1,  &endRotation1Event);
  startMove(rotation1);
  waitEvent(&endRotation1Event);

  setServoPos(LEFT_SERVO,OUT_LEFT_SERVO);

  sleep(2);
  onElectro(ELECTRO_REAR_LEFT_FLAG);
  sleep(1);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);


  // Retour au centre
  //******************
  rotation2 = defineRotationMovement(0.0);
  setMaxSpeed(rotation2,0,30.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation2Event,"END_ROTATION_2_EVENT");
  setEndMoveEvent(rotation2,  &endRotation2Event);
  startMove(rotation2);
  waitEvent(&endRotation2Event);


  sleep(5);

  // Rotation et depose du second drapeau
  //*************************************
  rotation3 = defineRotationMovement(-PI/2);
  setMaxSpeed(rotation3,30.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation3Event,"END_ROTATION_3_EVENT");
  setEndMoveEvent(rotation3,  &endRotation3Event);
  startMove(rotation3);  
  waitEvent(&endRotation3Event);

  setServoPos(RIGHT_SERVO,OUT_RIGHT_SERVO);
  
  sleep(2);

  onElectro(ELECTRO_REAR_RIGHT_FLAG);
  sleep(1);

  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);

  // Retour au centre
  //******************
  rotation4 = defineRotationMovement(0.0);
  setMaxSpeed(rotation4,30.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation4Event,"END_ROTATION_4_EVENT");
  setEndMoveEvent(rotation4,  &endRotation4Event);
  startMove(rotation4);  
  waitEvent(&endRotation4Event);


  sleep(5);


  // Avancement jusqu'a la planete centrale   
  //***************************************

  moveToBigStar = defineXYMovement(1400.0,0.0);
//  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveToBigStar,0.9);
  setEpsilon(moveToBigStar,10.0);
  setMaxSpeed(moveToBigStar,30.0,30.0,MAX_SPEED_PROPORTIONAL);
      //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveToBigStarEvent,"END_MOVE_TO_BIG_STAR_EVENT");
  setEndMoveEvent(moveToBigStar,  &endMoveToBigStarEvent);

  
  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");

  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

  sleep(2);

  armMotorOn();

  sleep(2);

  startMove(moveToBigStar);

//  motorOn(MOTOR_1,MOTOR_SENS_1);




  

  //  armMotor(ON);
//  setServoPos(ARM_SERVO,MAX_ARM_SERVO);

   waitEvent(&endMoveToBigStarEvent);

  printf("%lf - %lf - %lf\n", getX(),getY(),getTheta());

  debug_printf(DEBUG_MODULE_NAME,"End move to big star\n");

  sleep(2);

  onElectro(ELECTRO_ARM);
  
  sleep(20);
  
  onElectro(0);
  onElectro(1);
  onElectro(2);
  onElectro(3);
  onElectro(4);
  onElectro(6);
  onElectro(7);
  
  //armMotor(OFF);
//  motorOff(MOTOR_1);
      
//  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  setServoPos(ARM_SERVO,MIN_ARM_SERVO);

  debug_printf(DEBUG_MODULE_NAME,"END OF SCENARIO...\n");

  return 0;
}




