#include "robot.h"
#include "periodServer.h"
#include "motor.h"
#include "i2cServos.h"
#include <unistd.h>

#define DEBUG_MODULE_NAME MAIN
#include "debug.h"

#include "move.h"

#include "trajectory.h"
#include "fielddefinition.h"
#include "Intel.h"
#include "gotolib.h"
#include "flaglib.h"

#define PI 3.14159

#define CORRECTION_X -200
#define CORRECTION_Y 1050
/* 
 0 : no ennemy 
 1 : between star and ennemy home  
 2 : between star and ennemy planet 
*/
#define ENNEMY_KIND 1

void emergencyStopCallback(Event* event)
{
  printf("************************************************************************************************\n");
  printf("Callback emergency stop...\n");
  printf("************************************************************************************************\n");


  resetStop();

  lcd_clear();
  lcd_printf(" Emergency stop\n Program halted\n");
  sleep(2);
  exit(0);
}

int main()
{
  periodicTask* jackOn;
  periodicTask* jackOff;
  Event* jackOffEvent;
  Event* jackOnEvent;

  periodicTask* frontBumperAcquisition;
  Event* frontBumperEvent;
  periodicTask* frontBumperLeftAcquisition;
  Event* frontBumperLeftEvent;
  Event frontBumperFrontLeftEvent;
  periodicTask* frontBumperRightAcquisition;
  Event* frontBumperRightEvent;
  Event bumperEvent;

  periodicTask* switch12VAcquisition;
  Event* switch12VEvent;

  MovementDescriptor* moveToBigStar;
  Event endMoveToBigStarEvent;

  MovementDescriptor* moveBack;
  Event endMoveBackEvent;

  MovementDescriptor* rotation1;
  Event endRotation1Event;

  MovementDescriptor* rotation2;
  Event endRotation2Event;

  MovementDescriptor* rotation3;
  Event endRotation3Event;

  MovementDescriptor* rotation4;
  Event endRotation4Event;

  Point *splinePoints;
  Point directionDepart;
  Point directionFin;
  MovementDescriptor* spline;
  
  Event pwmLimitationEvent;

  Event stopEvent;
  Event stopEvent2;
  
  int launchBigStarPMIOK = 1; // we are optimistic

  debug_printf(DEBUG_MODULE_NAME,"Start soft\n");

  shared_initModule();

  i2c_initModule();

  INIThctl();

  move_initModule();
  
  electro_initModule();

  relay_initModule();

  perServer_initModule();

  trajectory_initModule();
  
  intel_init_module ();

  lcd_initModule();
  lcd_clear();

  /* definition des modes d'approche et des zones dangereuses */
  switch(ENNEMY_KIND){
  case 0 : 
    FieldDefinition_Alone();
    break;
  case 1 :
    FieldDefinition_Default();
    break;
  case 2:
    FieldDefinition_JustStar();
      break;
  case 3:
    FieldDefinition_RightEnnemy();
    break;
  case 4:
    FieldDefinition_LeftEnnemy();
    break;
  case 5:
    FieldDefinition_FileLoad();
    break;
  default:
    FieldDefinition_Default();
    break;
  }
  debug_printf(DEBUG_MODULE_NAME,"BEGIN of SCENARIO\n");
 
  setServoPos(ARM_SERVO,MIN_ARM_SERVO);
  setServoPos(RIGHT_SERVO,IN_RIGHT_SERVO);
  setServoPos(LEFT_SERVO,IN_LEFT_SERVO);

  rentreAllFlag();
 
  //motorOff(MOTOR_ARM);
//armMotor(OFF);

  // Gestion 12V 

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 0<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);

  lcd_clear();
  lcd_printf("Unlock Emergency\n      Stop\n");
  
  waitEvent(switch12VEvent);
  deletePeriodicAcquisition(switch12VAcquisition);

  switch12VAcquisition = createPeriodicAcquisition(I2C_12V_SLAVE, 1<<I2C_12V_BIT, 1<<I2C_12V_BIT,500000);
  switch12VEvent = i2c_getEvent(switch12VAcquisition);
  createCallback(switch12VEvent, emergencyStopCallback);
  

  // Gestion START JACK
  //******************
  jackOn  = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 1 << I2C_STARTJACK_BIT, 1 << I2C_STARTJACK_BIT, 100000);
  jackOff = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 0                     , 1 << I2C_STARTJACK_BIT, 100000);
  jackOffEvent = i2c_getEvent(jackOff);
  jackOnEvent = i2c_getEvent(jackOn);

  // Position de dpart, pas tres bien ici, mais on evite d'attendre la jack
  setRobotPos(-200,1050,0);
#if 0
  lcd_clear();
  lcd_printf("\nWaiting jack off\n");
  waitEvent(jackOffEvent);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack in\n");
  waitEvent(jackOnEvent);

  // Position de dpart
  setRobotPos(-200,1050,0);

  lcd_clear();
  lcd_printf("Robot ready\nWaiting jack off\n");
  waitEvent(jackOffEvent);

#endif

  //  lcd_clear();

  deletePeriodicAcquisition(jackOn);
  deletePeriodicAcquisition(jackOff);

  // while(1) { sleep(1); }


  // Avancement jusqu'a la planete centrale   
  //***************************************

  frontBumperAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_SLAVE, 
					    1 << I2C_FRONT_BUMPER_BIT , 
					    1 << I2C_FRONT_BUMPER_BIT, 50000);

  frontBumperEvent = i2c_getEvent(frontBumperAcquisition);

  frontBumperLeftAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_LEFT_SLAVE, 
					          0 << I2C_FRONT_BUMPER_LEFT_BIT , 
					          1 << I2C_FRONT_BUMPER_LEFT_BIT, 50000);

  frontBumperLeftEvent = i2c_getEvent(frontBumperLeftAcquisition);

  printf ("*****Create event ***\n");
  createEvent(&frontBumperFrontLeftEvent,"OR_FRONT_FRONT_LEFT");
  orEvents(frontBumperEvent, frontBumperLeftEvent, &frontBumperFrontLeftEvent);

  frontBumperRightAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_RIGHT_SLAVE, 
					    0 << I2C_FRONT_BUMPER_RIGHT_BIT , 
					    1 << I2C_FRONT_BUMPER_RIGHT_BIT, 50000);

  frontBumperRightEvent = i2c_getEvent(frontBumperRightAcquisition);

   printf ("*****Create event ***\n");
  createEvent(&bumperEvent,"OR_FRONT_FRONT_LEFT");
  orEvents(frontBumperRightEvent, &frontBumperFrontLeftEvent, &bumperEvent);
 

  
  
  moveToBigStar = defineXYMovement(1360.0+CORRECTION_X,0.0+CORRECTION_Y);
//  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setPIDParams(moveToBigStar,0.9);
  setEpsilon(moveToBigStar,10.0);
      //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveToBigStarEvent,"END_MOVE_TO_BIG_STAR_EVENT");
  setEndMoveEvent(moveToBigStar,  &endMoveToBigStarEvent);
  // bumper management
  
//  frontBumperEvent = i2c_getEvent(frontBumperAcquisition);

  createEvent(&stopEvent,"STOP_EVENT");
//  orEvents(frontBumperEvent, &endMoveToBigStarEvent, &stopEvent);
  orEvents(&bumperEvent, &endMoveToBigStarEvent, &stopEvent);

  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveToBigStar);

//  motorOn(MOTOR_ARM,  MOTOR_ARM_UP);
  setServoPos(ARM_SERVO,MAX_ARM_SERVO);
  armMotorOn();

  //  waitEvent(&endMoveToBigStarEvent);

  waitEvent(&stopEvent);
  debug_printf(DEBUG_MODULE_NAME,"0x%x - 0x%x\n",stopEvent.signalingEvent,frontBumperEvent);
    //if (stopEvent.signalingEvent == frontBumperEvent)
  if (stopEvent.signalingEvent == &bumperEvent)
    {
   //   if (getX() < 850.0)
	{
	  //	  resetStop();
	  // reduce the max speed
	  setMaxSpeed(moveToBigStar, 20, 20, MAX_SPEED_PROPORTIONAL);
	  setFinalK(moveToBigStar, 0.5);

	  usleep(500000);

	  createEvent(&pwmLimitationEvent,"PWM LIMITATION");
	  createEvent(&stopEvent2,"STOP_EVENT_2");
	  orEvents(&pwmLimitationEvent, &endMoveToBigStarEvent, &stopEvent2);

	  setPWMLimitationEvent(&pwmLimitationEvent); 
	  setPWMLimitation(50);
	  enablePWMLimitation();
	  
	  waitEvent(&stopEvent2);

	  if (stopEvent2.signalingEvent == &pwmLimitationEvent)
	    {
	      resetStop();
	      launchBigStarPMIOK=0;
	    }

	}
      //stopMove();
    }

  printf("%lf - %lf - %lf\n", getX(),getY(),getTheta());


  debug_printf(DEBUG_MODULE_NAME,"End move to big star\n");
  deletePeriodicAcquisition(frontBumperAcquisition);
  
  setPWMLimitation(255);

  onElectro(ELECTRO_RIGHT_PMI);
  onElectro(ELECTRO_LEFT_PMI);

   if (launchBigStarPMIOK==1)
    {
      setServoPos(ARM_SERVO,MAX_ARM_SERVO);
      
      //sleep(2);
      dropFlagOnPlanet(FLAG_STAR, PLANET_STAR);
      //electroOn(ELECTRO_ARM);
      //sleep(1);
    }

//  motorOff(MOTOR_ARM);
  
  
      
  setServoPos(ARM_SERVO,MIN_ARM_SERVO); 

  // Recul jusqu'au point au centre des 2 planetes "chez nous"
  //**********************************************************
  moveBack = defineXYMovement(1050.0+CORRECTION_X,0.0+CORRECTION_Y);
  setRefPoint(moveBack,-5.0,0.0); //-1.0
  setPIDParams(moveBack,0.7);  //0.1
  createEvent(&endMoveBackEvent,"END_MOVE_BACK_EVENT");
  setEndMoveEvent(moveBack,  &endMoveBackEvent);

  debug_printf(DEBUG_MODULE_NAME,"Start move back\n");
  startMove(moveBack);
  waitEvent(&endMoveBackEvent);

  printf("X : %lf - Y : %lf - Theta : %lf\n", getX(), getY(), getTheta());;

 // exit(0);

  prepareFlag(FLAG_MY_RIGHT);
  //setServoPos(RIGHT_SERVO,MAX_RIGHT_SERVO);
  //setServoPos(RIGHT_SERVO,MAX_RIGHT_SERVO);
  
    // Rotation et depose du second drapeau
  //*************************************
  rotation3 = defineRotationMovement(-(PI/2)+(PI/6)+(PI/8));
  setMaxSpeed(rotation3,127.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation3Event,"END_ROTATION_3_EVENT");
  setEndMoveEvent(rotation3,  &endRotation3Event);
  startMove(rotation3);  
  waitEvent(&endRotation3Event);

  //exit(0);

  //sleep(3);
  dropFlagOnPlanet(FLAG_MY_RIGHT, PLANET_MY_RIGHT);
  //electroOn(ELECTRO_REAR_RIGHT_FLAG);
  //sleep(2);
  rentreFlag(FLAG_MY_RIGHT);
  //setServoPos(RIGHT_SERVO,MIN_RIGHT_SERVO);
  //setServoPos(RIGHT_SERVO,MIN_RIGHT_SERVO);


//  sleep(5);

  // Retour au centre
  //******************
  rotation4 = defineRotationMovement(0.0);
  setMaxSpeed(rotation4,127.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation4Event,"END_ROTATION_4_EVENT");
  setEndMoveEvent(rotation4,  &endRotation4Event);
  startMove(rotation4);  
  waitEvent(&endRotation4Event);
 
  prepareFlag(FLAG_MY_LEFT);
  //setServoPos(LEFT_SERVO,MIN_LEFT_SERVO);
  //setServoPos(LEFT_SERVO,MIN_LEFT_SERVO);

  // Rotation et depose du premier drapeau
  //**************************************
  rotation1 = defineRotationMovement((PI/2)-(PI/6)-(PI/8));
  setMaxSpeed(rotation1,0.0,127.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation1Event,"END_ROTATION_1_EVENT");
  setEndMoveEvent(rotation1,  &endRotation1Event);
  startMove(rotation1);
  waitEvent(&endRotation1Event);
  
  //sleep(3);
  dropFlagOnPlanet(FLAG_MY_LEFT, PLANET_MY_LEFT);
  //electroOn(ELECTRO_REAR_LEFT_FLAG);
  //sleep(2);;
  rentreFlag(FLAG_MY_LEFT);
  //setServoPos(LEFT_SERVO,MAX_LEFT_SERVO);
  //setServoPos(LEFT_SERVO,MAX_LEFT_SERVO);


//  sleep(5);

  // Retour au centre
  //******************
  rotation2 = defineRotationMovement(0.0);
  setMaxSpeed(rotation2,0.0,127.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation2Event,"END_ROTATION_2_EVENT");
  setEndMoveEvent(rotation2,  &endRotation2Event);
  startMove(rotation2);
  waitEvent(&endRotation2Event);

  //setServoPos(RIGHT_SERVO,MIN_RIGHT_SERVO);
  //setServoPos(RIGHT_SERVO,MIN_RIGHT_SERVO);


  
   

   intel_activate(); /* pour ce qui reste */
  
#if 0
  // Mouvement jusqu'a une des planetes avec Velcro
  //***********************************************
  splinePoints = (Point*) malloc(3*sizeof(Point));
  splinePoints[0].x = getX();
  splinePoints[0].y = getY();
  splinePoints[1].x = 1200.0;
  splinePoints[1].y = 200.0;
  splinePoints[2].x = 16.0;
  splinePoints[2].y = 600.0;
  directionDepart.x = 0;
  directionDepart.y = 10;
  directionFin.x = 10;
  directionFin.y = 10;
  
  spline = defineSplineMovement(directionDepart, directionFin, splinePoints, 3);
  setPIDParams(spline,0.3);
  startMove(spline);
#endif

  sleep(30);

  debug_printf(DEBUG_MODULE_NAME,"END OF SCENARIO...\n");

  return 0;
}




