#include <xmlmemory.h>
#include <parser.h>
#include <stdio.h>
#include <dlfcn.h>

#include "splashScreen.h"
#include "drawingArea.h"
#include "comManager.h"

xmlDocPtr doc;

void parseNetwork(xmlDocPtr doc, xmlNodePtr cur)
{
  char dummy[200];
  int port;
  
  cur = cur->xmlChildrenNode;
  
  while (cur != NULL)
    {

      if (!strcmp(cur->name,"host"))
	{
	  printf("Host : %s\n",xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	  comManager_setHost(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	}

      if (!strcmp(cur->name,"port"))
	{
	  printf("Port : %s\n",xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	  port = atoi(xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	  printf("%d\n",port);
	  comManager_setPort(port);
	}
      cur = cur->next;
    }
}

void parsePlayground(xmlDocPtr doc, xmlNodePtr cur)
{
  //cur = cur->xmlChildrenNode;
  setXMLBackground(doc,cur);
  printf("doc : 0x%x - node : 0x%x\n",doc,cur);
  printf("Playground : %s\n",xmlGetProp(cur, "xMax"));
}

void parseModule(xmlDocPtr doc, xmlNodePtr cur)
{
  char* moduleName;
  void *dlHandle;
  void (*procPtr)();
  char dummy[200];

  printf("Parse module\n");
  cur = cur->xmlChildrenNode;
  while (cur != NULL)
    {
      printf("%s\n",cur->name);
      if (!strcmp(cur->name,"name"))
	{
	  sprintf(dummy,"Load module (%s)...\n",xmlNodeListGetString(doc, cur->xmlChildrenNode, 1));
	  splashScreen_incProgress(0.1);
	  splashScreen_setLabel(dummy);
	}
      if (!strcmp(cur->name,"location"))
	{
	  moduleName=xmlNodeListGetString(doc, cur->xmlChildrenNode, 1);
	  if (!(dlHandle = dlopen(moduleName,RTLD_LAZY)))
	    {
	      printf("Error loading module : %s\n",dlerror());
	    }

	  printf("Init...\n");
	  procPtr = dlsym(dlHandle,"initModule");
	  if (!procPtr)
	    {
	      printf("Error getting address of initModule : %s\n",dlerror());
	    }

	  (*procPtr)();
	}
      cur = cur->next;
    }
}


int handleConfig()
{
  xmlNodePtr cur;
  xmlNsPtr ns;
  
  splashScreen_setLabel("Parsing config file...");

  doc = xmlParseFile("config.xml");
  
  if (doc == NULL) 
    {
      splashScreen_setLabel("Error parsing config.xml\n");
      return -1;
    };


  splashScreen_setLabel("Interpreting config file...");

  cur = xmlDocGetRootElement(doc);  

  ns = xmlSearchNsByHref(doc, cur, "http://www.gege.org/coreff/config/1.0");
  if (ns == NULL) {
    splashScreen_setLabel("This file is not a Config file !\n");
  }

  if (strcmp(cur->name, "config")) {
    splashScreen_setLabel("The root element is not Config !!!!\n");
  }


  cur = cur->xmlChildrenNode;
  while (cur != NULL)
    {
      if (!strcmp(cur->name,"module"))
	{
	  parseModule(doc,cur);
	}

      if (!strcmp(cur->name,"network"))
	{
	  parseNetwork(doc,cur);
	}

      if (!strcmp(cur->name,"playground"))
	{
	  parsePlayground(doc,cur);
	}

      cur = cur->next;
    }

//  xmlFreeDoc(doc);

  return 0;
}
