#include <gtk/gtk.h>  

#include <pthread.h>

#include <stdio.h>

#include "mainWindow.h"
#include "drawingPanel.h"
#include "gtkMutex.h"

static GtkWidget *window;
static GtkWidget *notebook;

static GtkWidget *logTextBox;

void delete_event (GtkWidget *widget, GdkEvent *event, gpointer *data)
{
  gtk_main_quit ();
}

void mainWindow_addNoteBookPage(GtkWidget *noteBookPage)
{
  pthread_mutex_lock(&mutexGTKLoop);
  gtk_container_add (GTK_CONTAINER (notebook), noteBookPage);
  pthread_mutex_unlock(&mutexGTKLoop);
}

void mainWindow_setNoteBookPageName(GtkWidget *noteBookPage, GtkWidget *label)
{
  pthread_mutex_lock(&mutexGTKLoop);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook), noteBookPage, label);
  pthread_mutex_unlock(&mutexGTKLoop);
}

void mainWindow_addLog(char* text)
{
  pthread_mutex_lock(&mutexGTKLoop);
  gtk_text_insert (GTK_TEXT (logTextBox), NULL, NULL, NULL, text, strlen (text));
  gtk_text_insert (GTK_TEXT (logTextBox), NULL, NULL, NULL, "\n", 1);
  pthread_mutex_unlock(&mutexGTKLoop);
}

void mainWindow_addLog_NM(char* text)
{
  gtk_text_insert (GTK_TEXT (logTextBox), NULL, NULL, NULL, text, strlen (text));
  gtk_text_insert (GTK_TEXT (logTextBox), NULL, NULL, NULL, "\n", 1);
}

void mainWindow_init()
{
  GtkWidget *label;
  GtkWidget *empty_notebook_page;
  GtkWidget *vbox;
  GtkWidget *v2box;
  GtkWidget *hbox;
  GtkWidget *drawingPanel;
//  GtkWidget *drawingArea;
//  GtkWidget *scrolledWindow;

  pthread_mutex_lock(&mutexGTKLoop);
  
  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_position(GTK_WINDOW(window),GTK_WIN_POS_CENTER);
  gtk_window_set_title (GTK_WINDOW (window), "Coreff Control Center");

  gtk_signal_connect (GTK_OBJECT (window), "delete_event",
		      GTK_SIGNAL_FUNC (delete_event), NULL);
  
  gtk_container_border_width (GTK_CONTAINER (window), 10);

  
  vbox=gtk_vpaned_new();
  gtk_container_add (GTK_CONTAINER (window), vbox);
  gtk_widget_show (vbox);

  hbox=gtk_hpaned_new();
//  gtk_container_add (GTK_CONTAINER (window), hbox);
  gtk_paned_add1(GTK_PANED (vbox), hbox);
  gtk_widget_show (hbox);


  // notebook
//  vbox = gtk_vbox_new (FALSE,0);
//  gtk_container_add (GTK_CONTAINER (hbox), vbox);
//  gtk_paned_add1(GTK_PANED (hbox), vbox);
//  gtk_widget_show (vbox);

  notebook = gtk_notebook_new ();
//  gtk_box_pack_start (GTK_BOX (vbox), notebook, FALSE, FALSE, 5);
  gtk_paned_add1(GTK_PANED (hbox), notebook);
  gtk_container_border_width (GTK_CONTAINER (notebook), 10);
  gtk_widget_show (notebook);

  v2box = gtk_hbox_new (FALSE, 0);
  logTextBox = gtk_text_new(gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 10.0, 100.0),
			    gtk_adjustment_new(0.0, 0.0, 100.0, 1.0, 10.0, 100.0));
  gtk_container_add (GTK_CONTAINER (v2box), logTextBox);
  gtk_widget_show (logTextBox);

  gtk_container_border_width (GTK_CONTAINER (v2box), 10);

//  gtk_box_pack_start (GTK_BOX (vbox), logTextBox, FALSE, FALSE, 5);
  gtk_paned_add2(GTK_PANED (vbox), v2box);
  gtk_widget_show (v2box);

  // playground
#if 0
  scrolledWindow = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledWindow),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_container_border_width (GTK_CONTAINER (scrolledWindow), 10);

  gtk_paned_add2(GTK_PANED (hbox), scrolledWindow);
  gtk_widget_show (scrolledWindow);

  drawingArea = drawingArea_init();
  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (scrolledWindow), drawingArea);
  drawingArea_show();
#endif

  drawingPanel = drawingPanel_init();
  gtk_paned_add2(GTK_PANED (hbox), drawingPanel);
  drawingPanel_show();


  pthread_mutex_unlock(&mutexGTKLoop);
}

void mainWindow_show()
{
  printf("mainWindow_show()\n");
  pthread_mutex_lock(&mutexGTKLoop);

  gtk_widget_show (window);
  pthread_mutex_unlock(&mutexGTKLoop);
}
