#include <stdio.h>

#include "mainWindow.h"

#include "gtkMutex.h"

#include "comManager.h"

#include "drawingArea.h"

#define PI 3.14159

GtkWidget *xValueLabel,*yValueLabel,*thetaValueLabel;

static Layer* drawingLayer;


static void createBitmap(Layer* layer)
{
  printf("Drawing layer\n");
  drawRectangle(layer,"blue", 1, 10,10,300,300);
}



void posCallBack(char* buffer)
{
  float x;
  float y;
  float theta;
  char dummy[50];

  printf("PosCallBack (%s)\n",buffer);

  sscanf(buffer,"POS X:%f Y:%f T:%f\n",&x,&y,&theta);
  
  printf("PosCallBack : x : %f - y : %f - theta : %f\n",x,y,theta);

  eraseLayer(drawingLayer);
  drawRobot(drawingLayer,1,x,y,theta);

  drawRobot(drawingLayer,0,x+100.0,y+100.0,theta+0.2);
  drawingAreaRedraw();

  sprintf(dummy,"%.2f",x);
  gtk_label_set_text(GTK_LABEL(xValueLabel),dummy);

  sprintf(dummy,"%.2f",y);
  gtk_label_set_text(GTK_LABEL(yValueLabel),dummy);

  sprintf(dummy,"%.2f",theta);
  gtk_label_set_text(GTK_LABEL(thetaValueLabel),dummy);

}

GtkWidget *createCoderFrame()
{
  GtkWidget *frame;
  GtkWidget *table;
  GtkWidget *xLabel, *yLabel, *thetaLabel;
  
  frame = gtk_frame_new(" Position ");
  
  gtk_widget_show (frame);

  table = gtk_table_new (2,3,TRUE);

  gtk_container_border_width (GTK_CONTAINER (table), 10);
  gtk_widget_show (table);

  gtk_container_add (GTK_CONTAINER (frame), table);

  xLabel = gtk_label_new ("X");
  gtk_widget_show (xLabel);
  gtk_table_attach (GTK_TABLE(table),
                    xLabel,
                    0,
                    1,
                    0,
                    1,
                    GTK_EXPAND,
                    GTK_EXPAND,
                    5,
                    5);

  yLabel = gtk_label_new ("Y");
  gtk_widget_show (yLabel);
  gtk_table_attach (GTK_TABLE(table),
                    yLabel,
                    0,
                    1,
                    1,
                    2,
                    GTK_EXPAND,
                    GTK_EXPAND,
                    5,
                    5);

  
  thetaLabel = gtk_label_new ("Theta");
  gtk_widget_show (thetaLabel);
  gtk_table_attach (GTK_TABLE(table),
                    thetaLabel,
                    0,
                    1,
                    2,
                    3,
                    GTK_EXPAND,
                    GTK_EXPAND,
                    5,
                    5);


  
  xValueLabel = gtk_label_new ("?");
  gtk_widget_show (xValueLabel);
  gtk_table_attach (GTK_TABLE(table), xValueLabel, 1, 2, 0, 1, GTK_EXPAND, GTK_EXPAND, 5, 5);

  yValueLabel = gtk_label_new ("?");
  gtk_widget_show (yValueLabel);
  gtk_table_attach (GTK_TABLE(table), yValueLabel, 1, 2, 1, 2, GTK_EXPAND, GTK_EXPAND, 5, 5);

  thetaValueLabel = gtk_label_new ("?");
  gtk_widget_show (thetaValueLabel);
  gtk_table_attach (GTK_TABLE(table), thetaValueLabel, 1, 2, 2, 3, GTK_EXPAND, GTK_EXPAND, 5, 5);

    
  return frame;
}

void initModule()
{
  GtkWidget *empty_notebook_page;
  GtkWidget *label;
  GtkWidget *softResetButton, *hardResetButton;
  GtkWidget *vbox;

  printf("Init module\n");

//  pthread_mutex_lock(&mutexGTKLoop);

  vbox = gtk_vbox_new (FALSE, 0);
  empty_notebook_page = vbox;
  gtk_container_border_width (GTK_CONTAINER (vbox), 10);
  gtk_widget_show (empty_notebook_page);

  gtk_box_pack_start (GTK_BOX (vbox), createCoderFrame(), FALSE, FALSE, 5);

  mainWindow_addNoteBookPage(empty_notebook_page);
  label = gtk_label_new ("POS");
  mainWindow_setNoteBookPageName(empty_notebook_page, label);


  /* drawing area */
  printf("Creating layer\n");
  drawingLayer=addDrawingLayer(createBitmap);


  registerNetCallBack("POS",posCallBack);
  
  mainWindow_addLog("POS module initialized...");


  
//  pthread_mutex_unlock(&mutexGTKLoop);
}
