#include <stdio.h>  // printf
#include <stdlib.h> // malloc
#include <unistd.h> // sleep
#include <math.h>

#include "robot.h"
#include "periodServer.h"
#include "robot_geometry.h"
#include "flaglib.h"
#include "Intel.h" // pour #define CANCEL
#include "motor.h"
#include "electro.h"
#include "i2cServos.h"
#include "i2cMapping.h"

#include "log.h"

#define DEBUG
#define LOG_DEBUG
//#ifdef LOG_DEBUG
//#define LOG_FILENAME "trajectory.log"
//FILE*fptr1;
//#endif
#define DEBUG_MODULE_NAME FLAGLIB
#include "debug.h"

int brasout=-1;
/******************************************************************************/
/*		FONCTIONS DES BRAS
/******************************************************************************/
void prepareFlag(int flagnbr)
{
  //TODO
  brasout=flagnbr;
  switch(flagnbr){
  case FLAG_STAR  :   
    armMotorOn();  
    setServoPos(ARM_SERVO,MAX_ARM_SERVO);  
      break;
  case FLAG_MY_RIGHT :
    setServoPos(RIGHT_SERVO,MAX_RIGHT_SERVO);
    break;
  case  FLAG_MY_LEFT:
    setServoPos(LEFT_SERVO,MAX_LEFT_SERVO);
    break;
  case   FLAG_ENNEMY_RIGHT:
  case   FLAG_ENNEMY_LEFT:
  case   FLAG_VELCRO_RIGHT:
  case   FLAG_VELCRO_LEFT :
    break;
  }
}

void rentreAllFlag()
{
  int i;
  for(i=0;i<BRAS_NUMBER;i++) 
    rentreFlag(i);
}
void rentreBrasOut()
{
  rentreFlag(brasout);
}
void rentreFlag(int flagnbr)
{
  switch(flagnbr){
  case FLAG_STAR  :  
    setServoPos(ARM_SERVO,MIN_ARM_SERVO); 
    break;
  case FLAG_MY_RIGHT :
    setServoPos(RIGHT_SERVO,MIN_RIGHT_SERVO);
    break;
  case  FLAG_MY_LEFT:
    setServoPos(LEFT_SERVO,MIN_LEFT_SERVO);
    break;
  case   FLAG_ENNEMY_RIGHT:
  case   FLAG_ENNEMY_LEFT:
  case   FLAG_VELCRO_RIGHT:
  case   FLAG_VELCRO_LEFT:
    break;
  }
}

void dropFlag(int flagnbr)
{
//  sleep(1);
  switch(flagnbr){
  case FLAG_STAR  :  
    onElectro(ELECTRO_ARM); // s'etteind tout seul
    break;
  case FLAG_MY_RIGHT :
    onElectro(ELECTRO_REAR_RIGHT_FLAG);
    break;
  case  FLAG_MY_LEFT:
    onElectro(ELECTRO_REAR_LEFT_FLAG);
    break;
  case   FLAG_ENNEMY_RIGHT:
    onElectro(ELECTRO_FRONT_RIGHT_FLAG);
    break;
  case   FLAG_ENNEMY_LEFT:
    onElectro(ELECTRO_FRONT_LEFT_FLAG);
    break; 
  case   FLAG_VELCRO_RIGHT:
    break;
  case   FLAG_VELCRO_LEFT:
    break;
  }
  BrasList[flagnbr].flagAvailable=FALSE;
  usleep(500000);

  printLog("drop flag %d \n",flagnbr/*,getX(),getY(),getTheta()*/);
//#ifdef LOG_DEBUG
//  fptr1=fopen(LOG_FILENAME,"a");
//  if(fptr1){
//    fprintf(fptr1,"drop flag %d \n",flagnbr/*,getX(),getY(),getTheta()*/);
//    fclose(fptr1);
//  }
//#endif
}

void dropFlagOnPlanet(int flagnbr, int planetnbr)
{
  dropFlag(flagnbr);
  PlanetIsDone(planetnbr);
}
int isFlagOnPlanet(int flagnbr, int planetnbr)
{
   return IsBrasOnPlanet(&BrasList[flagnbr],&Objectifs[planetnbr]);
}

int isFlagAvailable(int flagnbr)
{
  return IsBrasAvailable(&BrasList[flagnbr]);
}

void PlanetIsDone(int nbrplanet)
{
  SetPlanetDone(&Objectifs[nbrplanet]);
}
void SetPlanetDone(Objectif * planet)
{
  if(planet!=NULL)
    planet->Etat=CANCEL;
}
int IsPlanetDone(int nbrplanet)
{
  return (Objectifs[nbrplanet].Etat==CANCEL);
}

/*******************************************/
/* same functions but with pointers */
/*******************************************/
void PrepareBras(Bras *bras)
{
  if(bras==NULL) return;
  prepareFlag(bras->brasnbr);
}

void DropFlagWithBras(Bras * bras)
{
  if(bras==NULL) return;
  dropFlag(bras->brasnbr);
}

void RentreBras(Bras * bras)
{
  if(bras==NULL) return;
  rentreFlag(bras->brasnbr);
}

int IsBrasAvailable(Bras * bras)
{
   if(bras==NULL) return FALSE;
   return bras->flagAvailable;
}

int IsBrasOnPlanet(Bras* bras, Objectif *planet)
{
  double x,y;
   if(bras==NULL || planet==NULL) return FALSE;
   x = getX()+bras->r*cos(bras->theta+getTheta())-planet->Coordonnees.x;
   y = getY()+bras->r*sin(bras->theta+getTheta())-planet->Coordonnees.y;
#ifdef DEBUG
//   #ifdef LOG_DEBUG
//  fptr1=fopen(LOG_FILENAME,"a");
//  if(fptr1){
//    fprintf(fptr1,"IsBrasOnPlanet %d %d %lf %lf\n",bras->brasnbr, planet->PlaneteNb,x,y);
//    fclose(fptr1);
//  }
//   printLog("IsBrasOnPlanet %d %d %lf %lf\n",bras->brasnbr, planet->PlaneteNb,x,y);
//#endif
   printLog("IsBrasOnPlanet %d %d %lf %lf\n",bras->brasnbr, planet->PlaneteNb,x,y);
#endif
   return (sqrt(x*x+y*y)<planet->rayon-bras->rMinOnPlanet);
}


int currentbras=-1;
Bras* GetFirstBras(Objectif * planet)
{
  int i;
  if(planet==NULL) return NULL;
  #ifdef DEBUG
  printf("GetFirstBras\n");
#endif
  currentbras=-1;
  for(i=0;i<BRAS_NUMBER;i++){
#ifdef DEBUG
  printf("GetFirstBras :%d \n",i);
#endif
    if(planet->Drapeaux[i]>=0 && planet->Drapeaux[i]<BRAS_NUMBER && IsBrasAvailable(&(BrasList[planet->Drapeaux[i]]))){
    	currentbras = i;  
#ifdef DEBUG
  printf("GetFirstBras end\n");
#endif
	return &(BrasList[planet->Drapeaux[currentbras]]);
    }

  }
#ifdef DEBUG
  printf("GetFirstBras end NULL\n");
#endif
  return NULL;
}

Bras* GetNextBras(Objectif * planet)
{
  int i;
  if(planet==NULL) return NULL;
  for(i=currentbras+1;i<BRAS_NUMBER;i++){
    if(planet->Drapeaux[i]>=0 && planet->Drapeaux[i]<BRAS_NUMBER && IsBrasAvailable(&(BrasList[planet->Drapeaux[i]]))){
      currentbras = i;
      return &(BrasList[planet->Drapeaux[currentbras]]);
    }
  }
  return GetFirstBras(planet);
}

Bras* GetNearestBras(Objectif * planet)
{
  if(planet==NULL) return NULL;
  // TODO
  return GetFirstBras(planet);
}
