

#ifndef __LIGNES_KALMAN__
#define  __LIGNES_KALMAN__


#include "projet_kalman.h"
#include "lignes.h"
#include "which_line.c"
#include <math.h>
#include <stdio.h>


double mesuredist(x,y,alpha,Rc,alphac,a,b,c)
double x;
double y;
double alpha;
double Rc;
double alphac;
double a;
double b;
double c;
{
  double t11;
  double t12;
  double t14;
  double t1;

#ifdef DEBUG

  printf("x=%f,y=%f alpha=%f rc=%f ac=%f,a=%f b=%f c=%f",x,y,alpha,Rc,alphac,a,b,c);
#endif

  t1 = alpha+alphac;
  t11 = a*a;
  t12 = b*b;
   t14 = sqrt(t11+t12);
   return((a*(x+Rc*cos(t1))+b*(y+Rc*sin(t1))+c)/t14);
}



double dmdistdx(x,y,alpha,Rc,alphac,a,b,c)
double x;
double y;
double alpha;
double Rc;
double alphac;
double a;
double b;
double c;
{
  double t1;
  double t4;
  double t2;

  t1 = a*a;
  t2 = b*b;
  t4 = sqrt(t1+t2);
  return(a/t4);
}

double dmdistdy(x,y,alpha,Rc,alphac,a,b,c)
double x;
double y;
double alpha;
double Rc;
double alphac;
double a;
double b;
double c;
{
  double t1;
  double t4;
  double t2;

  t1 = a*a;
  t2 = b*b;
  t4 = sqrt(t1+t2);
  return(b/t4);
}


double dmdistda(x,y,alpha,Rc,alphac,a,b,c)
double x;
double y;
double alpha;
double Rc;
double alphac;
double a;
double b;
double c;
{
  double t10;
  double t9;
  double t12;
  double t2;

  t2 = alpha+alphac;
  t9 = a*a;
  t10 = b*b;
  t12 = sqrt(t9+t10);
  return((-a*Rc*sin(t2)+b*Rc*cos(t2))/t12);
}

/*

inline double fd2(x,rc,alpha,y,ac,a,b,c)
double x;
double rc;
double alpha;
double y;
double ac;
double a;
double b;
double c;
{
  double t1;
  double t11;
  double t12;
  double t13;

  t1 = alpha+ac;
  t11 = a*(x+rc*cos(t1))+b*(y+rc*sin(t1))+c;
  t11=t11*t11;
  t12 = a*a;
  t13 = b*b;
 
  return(t11/(t12+t13));
}
*/




inline double dist(struct ligne *  lig,struct capteur * capt,double x,double y,double alpha){

 return mesuredist(x,y,alpha,capt->R,capt->alpha,lig->a,lig->b,lig->c);
 

}
/*

inline double fd2dx(x,rc,alpha,y,ac,a,b,c)
double x;
double rc;
double alpha;
double y;
double ac;
double a;
double b;
double c;
{
  double t12;
  double t11;
  double t1;

  t1 = alpha+ac;
  t11 = a*a;
  t12 = b*b;
  return(2*(a*(x+rc*cos(t1))+b*(y+rc*sin(t1))+c)/(t11+t12)*a);
}



inline double fd2dy(x,rc,alpha,y,ac,a,b,c)
double x;
double rc;
double alpha;
double y;
double ac;
double a;
double b;
double c;
{
  double t1;
  double t11;
  double t12;

  t1 = alpha+ac;
  t11 = a*a;
  t12 = b*b;
  return(2*(a*(x+rc*cos(t1))+b*(y+rc*sin(t1))+c)/(t11+t12)*b);
}


inline double fd2da(x,rc,alpha,y,ac,a,b,c)
double x;
double rc;
double alpha;
double y;
double ac;
double a;
double b;
double c;
{
  double t11;
  double t6;
  double t2;
  double t1;
  double t12;

  t1 = alpha+ac;
  t2 = cos(t1);
  t6 = sin(t1);
  t11 = a*a;
  t12 = b*b;
  return(2*(a*(x+rc*t2)+b*(y+rc*t6)+c)/(t11+t12)*(-a*rc*t6+b*rc*t2));
}

*/

void calculJacobMesureLigne(struct ligne * l, struct capteur * capt){

  double x,y,theta,rc,ac,a,b,c,dmdxt,dmdyt,dmdat;

  jacobMesureLigne=Mtx_GetNewMatrix (1, 3);
 

 x=get( position,0,0);
 y=get( position,1,0);
 theta=get( position,2,0);

 rc=capt->R;
 ac=capt->alpha;

 a=l->a;
 b=l->b;
 c=l->c;

 dmdxt=dmdistdx(x,y,theta,rc,ac,a,b,c);
 dmdyt=dmdistdy(x,y,theta,rc,ac,a,b,c);  
 dmdat=dmdistda(x,y,theta,rc,ac,a,b,c);


 set( jacobMesureLigne,0,0,dmdxt);  //dm/dx
 set( jacobMesureLigne,0,1,dmdyt);
 set( jacobMesureLigne,0,2,dmdat);


}

//----------------------------------------------------
//  methode position capteur
//----------------------------------------------------

double xcapt(struct capteur * capt){
  double x,theta;
 x=get( position,0,0);
 theta=get( position,2,0);
 x=x+capt->R*cos(capt->alpha+theta);
   //y=get( position,1,0);

 return x;

}

double ycapt(struct capteur * capt){
  double y,theta;
 y=get( position,1,0);
 theta=get( position,2,0);
 y=y+capt->R*sin(capt->alpha+theta);
 return y;
   //x=get( position,1,0);

}

double alphacapt(struct capteur * capt){

return get( position,2,0);
}


//------------------------------------------------------
//   methode fusion mesure Ligne
//------------------------------------------------------


void fusionMesureLigne(double erreur,struct ligne * lig,struct capteur * capt){ // on passe sur la ligne ay+bx+c=0

 Mtx_Matrix * erreur_mesure, * mesure,*jacob_mesure;

 double x,y,theta;
 double rc,ac;
 double a,b,c;
 double m,dm;

 
 x=get( position,0,0);
 y=get( position,1,0);
 theta=get( position,2,0);
 
#ifdef DEBUG

 printf("position du robot : x=%f y=%f alpah=%f\n",x,y,theta);
#endif
 rc=capt->R;
 ac=capt->alpha;
 
 a=lig->a;
 b=lig->b;
 c=lig->c;

 //------calcul de la mesure attendue---------
 m=mesuredist(x,y,theta,rc,ac,a,b,c);
 
#ifdef DEBUG
  printf("estimation distance  la ligne %f\n",m);
#endif

  //printf("estimation position du capteur %f %f\n",xcapt(capt),ycapt(capt));
//-------------calcul du dm----------------------------
  

 dm=-m;
 //-----------application du kalman------------------------


 calculJacobMesureLigne(lig, capt);
#ifdef DEBUG
 printf("nouvelle jacobienne : ");
#endif

 Mtx_MatrixPrint(jacobMesureLigne);

 jacob_mesure=jacobMesureLigne; //(vecteurligne 3 colones)

 erreur_mesure=Mtx_GetNewMatrix(1,1); //(vecteurligne 1 colones)
 set(erreur_mesure,0,0,erreur*erreur);
 mesure=Mtx_GetNewMatrix(1,1); //(vecteurligne 1 colones)
 set(mesure,0,0,dm);

 //printf("calcul gain\n");
 calculGain(jacob_mesure, erreur_mesure, mesure);
 //printf("update state\n");
 generalised_updateState(mesure);
 //printf("update erreur\n");
 update_erreur(jacob_mesure);


Mtx_DeleteMatrix(erreur_mesure);
Mtx_DeleteMatrix(mesure);
Mtx_DeleteMatrix(jacob_mesure);

//----------------fin du filtre de kalman-----------


}




#endif 









