/*********************************************************************/
/* Mtx : Matrix Package                                              */
/* Description : Header File                                         */
/* ----------------------------------------------------------------- */
/* Author : Benigne DEPREY & Laurent SAINT-MARCEL                    */
/* Date   : 01-Mar-2000                                              */
/*********************************************************************/

/*-------------------------------------------------------------------*/
/*     WARNING - WARNING - WARNING - WARNING - WARNING               */
/*    May return wrong values for non inversible matrix              */
/*-------------------------------------------------------------------*/
#ifndef __MATRICE_H_
#define __MATRICE_H_
#ifndef NULL
#define NULL 0
#endif
/*-------------------------------------------------------------------*/
/* Nested Includes                                                   */
/*-------------------------------------------------------------------*/
//#include <assert.h>
//#include <stdlib.h>

/*-------------------------------------------------------------------*/
/* Nested Includes                                                   */
/*-------------------------------------------------------------------*/
#define cMANAGER_SIZE  30
#define cMATRIX_SIZE   3

/*-------------------------------------------------------------------*/
/* Structure Declarations                                            */
/*-------------------------------------------------------------------*/
//typedef struct Mtx_MatrixStruct Mtx_Matrix;

/*********************************************************************/
/* Mtx_MatrixStruct                                                  */
/*********************************************************************/
typedef struct Mtx_MatrixStruct {
  struct Mtx_MatrixStruct* next;
  int         nbLine;
  int         nbColumn;
  double      tab[3][3];
} Mtx_Matrix;

/*********************************************************************/
/* Mtx_StorageStruct                                                 */
/*********************************************************************/
typedef struct Mtx_StorageStruct {
  struct Mtx_StorageStruct* next;
  Mtx_Matrix*  mtx;
} Mtx_Storage;

/*********************************************************************/
/* Mtx_ManagerStruct                                                 */
/*********************************************************************/
typedef struct Mtx_ManagerStruct {
  int         managerSize;
  int         storageLeft;

  Mtx_Matrix  storageBuffer[cMANAGER_SIZE];
  Mtx_Matrix* freeStorage;
} Mtx_Manager;


/*-------------------------------------------------------------------*/
/* Global Variable Declaration                                       */
/*-------------------------------------------------------------------*/
Mtx_Manager theManager;


/*-------------------------------------------------------------------*/
/* Exported Procedure Declaration                                    */
/*-------------------------------------------------------------------*/
void        Mtx_ManagerInit  ();
void        Mtx_DeleteMatrix (Mtx_Matrix* mtx);
Mtx_Matrix* Mtx_GetNewMatrix (int nbLine, int nbColumn);

void        Mtx_MatrixPrint  (Mtx_Matrix* mtx);
void        Mtx_CreateIdent  (Mtx_Matrix* mtx);

Mtx_Matrix* Mtx_CoeffProd    (Mtx_Matrix* mtx, double lambda);

Mtx_Matrix* Mtx_Product      (Mtx_Matrix* mtx1, Mtx_Matrix* mtx2);
Mtx_Matrix* Mtx_Sum          (Mtx_Matrix* mtx1, Mtx_Matrix* mtx2);
Mtx_Matrix* Mtx_Substract    (Mtx_Matrix* mtx1, Mtx_Matrix* mtx2);
Mtx_Matrix* Mtx_AddIdent     (Mtx_Matrix* mtx);
Mtx_Matrix* Mtx_Neg          (Mtx_Matrix* mtx);
Mtx_Matrix* Mtx_Inverse      (Mtx_Matrix* mtx);
Mtx_Matrix* Mtx_Transpose    (Mtx_Matrix* mtx);
Mtx_Matrix* Mtx_SubToIdent   (Mtx_Matrix* mtx);

Mtx_Matrix* Mtx_Copy         (Mtx_Matrix* mtx);


/*-------------------------------------------------------------------*/
/* Inline Procedure Declaration                                      */
/*-------------------------------------------------------------------*/
static inline
double get(Mtx_Matrix *mtx,int i,int j) {
  return mtx->tab[i][j];
}

static inline
void set(Mtx_Matrix *mtx,int i,int j,double v) {
  mtx->tab[i][j]=v;
}

#endif // __MATRICE_H_
