#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdarg.h>

#include "linux/i2c.h"
#include "linux/i2c-dev.h"
#include <stdio.h>

#include <errno.h>

#include "lcd.h"

#define OK 1
#define I2C_ERROR 0
#define nbEssais 10

#define RW 0x01
#define RS 0x02
#define LCDInstructionMode 0x00
#define LCDDisplayMode 0x02

int LCDligneActuelle = 0;
int LCDcolActuelle = 0;
char LCDbuffer[32];

/* plus ou moins interne: */
/* a ne pas utiliser a l'exterieur */
int LCDwrite(char buff);
int LCDwrite8bit(char instr, char data);
int LCDwrite4bit(char instr, char data);
int LCDsetCursor(int ligne, int col);
int LCDeffacerResteLigne();
int LCDremonterLigne();

int LCDwrite(char buff)
     /* il faut envoier les signaux, ensuite les valider (par enable),
	les laisser, retirer le enable, et encore attendre 
	un petit peu */

    /* 7: D7  D3
       6: D6  D2
       5: D5  D1
       4: D4  D0
       3: E
       2: RW
       1: RS 
       0: -
    */
{
  int i, etat;
  char buf[2];

  buf[0]=buff;
  etat = 0;
  i = 0;

  i2cSend(i2c_LCD_addr,buf[0],0xFF);

  buf[0] = buff | 0x08;   /*on met le enable a 1*/
  etat = 0;
  i = 0;

  i2cSend(i2c_LCD_addr,buf[0],0xFF);
  
  buf[0]=buff;
  etat = 0;
  i = 0;
  
  i2cSend(i2c_LCD_addr,buf[0],0xFF);
  
  return OK;
}

int LCDwrite8bit(char instr, char data)
     /* instr:
	  xxxxxx(RS)(RW)
        data:
	  d7....d0 */
{
    /* 7: D7  D3
       6: D6  D2
       5: D5  D1
       4: D4  D0
       3: E //c'est LCDwrite qui s'en occuppe (doit etre 0!)
       2: RW
       1: RS 
       0: -
    */
  char tmp;
  tmp = 0x00;
  if (instr & RS) tmp = tmp | 0x02;
  if (instr & RW) tmp = tmp | 0x04;
  data = 0xF0 & data;
  tmp = tmp | data;
  return LCDwrite(tmp);
}

int LCDwrite4bit(char instr, char data)
     /* instr:
	  xxxxxx(RS)(RW)
        data:
	  d7....d0  */
{
    /* 7: D7  D3
       6: D6  D2
       5: D5  D1
       4: D4  D0
       3: E //c'est LCDwrite qui s'en occuppe (doit etre 0!)
       2: RW
       1: RS 
       0: -
    */
  char tmp;
  char sauveData;
  sauveData = data;
  tmp = 0x00;
  if (instr & RS) tmp = tmp | 0x02;
  if (instr & RW) tmp = tmp | 0x04;
  data = 0xF0 & data;
  tmp = tmp | data;
  if (LCDwrite(tmp) != OK) return I2C_ERROR; /*les premiers quatre bits*/

  tmp = tmp & 0x0F;
  sauveData = sauveData << 4;
  sauveData = sauveData & 0xF0;
  tmp = tmp | sauveData;
  return LCDwrite(tmp); /*les dernieres quatre bits*/
}

int lcd_initModule()
     /* on passe d'abord en mode 8bit pour ensuite se (re)mettre
	en mode 4bit. fait encore quelques initialisations */
{
  char instr, data;
  int adapter_nr = 0; /* probably dynamically determined */
  char filename[20];
  char buf[3];
   
  instr = LCDInstructionMode;
  data = 0x38;  /*function set: 8bits(, 2lignes, 5x7dots) */

  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);


/* 
 if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000); 
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000); 
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000); 
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000); 
  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);
*/

  data = 0x28;  /*function set: 4bits, 2lignes, 5x7dots */
  //gege usleep(10000);
  LCDwrite8bit(instr, data);
  //gege usleep(10000);
  LCDwrite4bit(instr, data);

/*  if (LCDwrite8bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(8000);
  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);*/

  /* display off */
  data = 0x08;
  LCDwrite8bit(instr, data);

/*  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;*/

  /* entry mode */
  data = 0x06;
  LCDwrite8bit(instr, data);
  
/*  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;*/

  /* clear display */
  data = 0x01;
/*  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;*/
  LCDwrite4bit(instr, data);
  //gege usleep(10000);

  /* display on */
  data = 0x0C;
/* if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;*/
  LCDwrite4bit(instr, data);

  /* return home! */
  data = 0x02;
/*  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;*/
  LCDwrite4bit(instr, data);
  //gege usleep(10000);

  return OK;
}

int lcd_cleanUpModule()
{
}


int lcd_clear()
     /* efface LCD mais modifie pas LCDcolActuelle et LCDligneActuelle */
     /* change evidemment aussi le cursor! */
{
  char instr, data;
  instr = LCDInstructionMode;
  data = 0x01;
  if (LCDwrite4bit(instr, data) != OK) return I2C_ERROR;
  //gege usleep(2000);
  return OK;
}

int LCDsetCursor(int ligne, int col)
     /* se positionne sur ligne, col */
     /* mais!!! ne modifie pas LCDcolActuelle et LCDligneActuelle */
{
  char instr, data;
  instr = LCDInstructionMode;
  if (ligne == 0)
    data = 0x80 + col;
  else if (ligne == 1)
    data = 0xC0 + col;
  if (col < 16)
    return LCDwrite4bit(instr, data);
  return OK;
}

int LCDeffacerResteLigne()
     /* efface la reste de la ligne (depuis la position ou on est, */
     /* et! change le cursor!! */
     /* change pas LCDligneActuelle, LCDcolActuelle */
     /* modifie LCDbuffer: on met les blancs! */
{
  int i;
  char instr, data;
  if (LCDligneActuelle > 1 || LCDcolActuelle > 15) return OK;

  instr = LCDDisplayMode;
  
  for (i = LCDcolActuelle; i < 16; i++)
    {
      if (LCDwrite4bit(instr, ' ') != OK) return I2C_ERROR;
      LCDbuffer[LCDligneActuelle*16 + i] = ' ';
    }
  return OK;
}

int LCDremonterLigne()
     /* copy la deuxieme ligne dans la premiere */
     /* change! le cursor!!! */
     /* change pas LCDligneActuelle, LCDcolActuelle */
     /* modifie le LCDbuffer: premiere ligne devient deuxieme ligne */
     /* et deuxieme ligne devient vide! */
{
  int i, debut, fin;
  char instr;

  LCDsetCursor(0,0);

  instr = LCDDisplayMode;

  for (i = 0; i < 16; i++)
    {
      LCDbuffer[i] = LCDbuffer[i+16];
      LCDbuffer[i+16] = ' ';
      if (LCDwrite4bit(instr, LCDbuffer[i]) != OK) return I2C_ERROR;
    }
  return OK;
}

int LCDprintf2(char *message)
{
  int i=0, j, ligneVide;
  char instr, data;
  while (message[i] != 0)
    {
      if (message[i] == '\n') /* saut de ligne */
	{
	  if (LCDligneActuelle == 2)
	    {
      	      if (lcd_clear() != OK) return I2C_ERROR;
	      /* si la deuxieme ligne etait deja vide, alors */
	      /* pas la peine de la recopier! */
	      ligneVide = 1;
	      for (j = 16; j < 32; j++)
		if (LCDbuffer[j] != ' ') ligneVide = 0;
	      if (ligneVide == 0) if (LCDremonterLigne() != OK) return I2C_ERROR;

	      LCDligneActuelle = 2;
	      LCDcolActuelle = 0;
	    }
	  else
	    {
	      if (LCDeffacerResteLigne() != OK) return I2C_ERROR;
	      LCDligneActuelle++;
	      LCDcolActuelle = 0;
	      if (LCDsetCursor(1,0) != OK) return I2C_ERROR;
	    }
	}
      else
	{
	  if (LCDligneActuelle == 2)
	    {
	      if (LCDremonterLigne() != OK) return I2C_ERROR;
	      if (LCDsetCursor(1,0) != OK) return I2C_ERROR;
	      LCDligneActuelle = 1;
	      LCDcolActuelle = 0;
	    }
	  if (LCDcolActuelle < 16) {
	    instr = LCDDisplayMode;
	    if (LCDwrite4bit(instr, message[i]) != OK) return I2C_ERROR;
	    LCDbuffer[LCDligneActuelle*16 + LCDcolActuelle] = message[i];
	    LCDcolActuelle++;
	  }
	}
      i++;
    }
  return OK;
}

int lcd_printf(char *fmt, ...)
{
  char message[256];
  va_list argp;
  va_start(argp, fmt);
  vsprintf(message, fmt, argp);
  va_end(argp);
  LCDprintf2(message);
}

int lcd_setText(int ligne, int col, char *message)
{
  if (ligne < 2 && col < 16)
    {
      LCDligneActuelle = ligne;
      LCDcolActuelle = col;
      if (LCDsetCursor(ligne, col) != OK) return I2C_ERROR;
      return lcd_printf(message);
    }
}
	  







