#include <stdio.h>  // printf
#include <stdlib.h> // malloc
#include <unistd.h> // sleep
#include <math.h>

#include "robot.h"
#include "trajectory.h"
#include "collision.h"
#include "periodServer.h"

#include "move.h"
#include "robot_geometry.h"
#include "i2cMapping.h"
#include "gotolib.h"
#include "flaglib.h"

#define GOTOLIB_MAX_SPEED_ROTATION  50.
#define GOTOLIB_MAX_SPEED_SPLINE       50.
#define GOTOLIB_SPEED_LINEAR_MVT       25.
#define GOTOLIB_SPEED_COLLISION          25.
#define MAX_OUT_OF_PLANET_RETRY         2

#define DEBUG
#define LOG_DEBUG

#ifdef LOG_DEBUG
#define LOG_FILENAME "trajectory.log"
FILE*fptr1;
#endif

int mysign(double a){
  if(a<0) return -1; 
  else return 1; 
}

periodicTask* frontBumperAcquisition;
  Event* frontBumperEvent;
periodicTask* frontBumperLeftAcquisition;
  Event* frontBumperLeftEvent;
  Event  bumperEvent0;
periodicTask* frontBumperRightAcquisition;
  Event* frontBumperRightEvent;
  Event bumperEvent1;
periodicTask* rearBumperAcquisition;
  Event* rearBumperEvent;
  Event bumperEvent2;
periodicTask* rearBumperLeftAcquisition;
  Event* rearBumperLeftEvent;
  Event bumperEvent3;
periodicTask* rearBumperRightAcquisition;
  Event* rearBumperRightEvent;
  Event bumperEvent;

#define DEBUG_MODULE_NAME GOTOLIB
#include "debug.h"

Point *splinePoints;
int initgotolib=0;
Objectif * currentTarget=NULL;


void init_gotoLibModule()
{
   splinePoints = (Point*) malloc(NBR_POINT*sizeof(Point));
   collision_initModule();
   CollisionSetParameters(GOTOLIB_SPEED_COLLISION, 3. /*reduced factor*/, 70./*distance*/);
   initgotolib=1;
}

unsigned char thetaSup(double x, double y, double theta, void* params)
{
  double thetaRef = *((double*)params);
  return theta > thetaRef;
}

void CreateBumperEvents()
{
  /* front */
  frontBumperAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_SLAVE, 
					    1 << I2C_FRONT_BUMPER_BIT , 
					    1 << I2C_FRONT_BUMPER_BIT, 50000);

  frontBumperEvent = i2c_getEvent(frontBumperAcquisition);

  /* front left */
  frontBumperLeftAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_LEFT_SLAVE, 
					          0 << I2C_FRONT_BUMPER_LEFT_BIT , 
					          1 << I2C_FRONT_BUMPER_LEFT_BIT, 50000);

  frontBumperLeftEvent = i2c_getEvent(frontBumperLeftAcquisition);

  createEvent(&bumperEvent0,"OR_FRONT_FRONT_LEFT");
  orEvents(frontBumperEvent, frontBumperLeftEvent, &bumperEvent0);

  /* front right */
  frontBumperRightAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_RIGHT_SLAVE, 
					    0 << I2C_FRONT_BUMPER_RIGHT_BIT , 
					    1 << I2C_FRONT_BUMPER_RIGHT_BIT, 50000);

  frontBumperRightEvent = i2c_getEvent(frontBumperRightAcquisition);

  createEvent(&bumperEvent1,"OR_FRONT_ALL");
  orEvents(frontBumperRightEvent, &bumperEvent0, &bumperEvent1);

 /* rear */
  rearBumperAcquisition = createPeriodicAcquisition(I2C_REAR_BUMPER_SLAVE, 
					    1 << I2C_REAR_BUMPER_BIT , 
					    1 << I2C_REAR_BUMPER_BIT, 50000);

  rearBumperEvent = i2c_getEvent(rearBumperAcquisition);

  /* rear left */
  rearBumperLeftAcquisition = createPeriodicAcquisition(I2C_REAR_BUMPER_LEFT_SLAVE, 
					          0 << I2C_REAR_BUMPER_LEFT_BIT , 
					          1 << I2C_REAR_BUMPER_LEFT_BIT, 50000);

  rearBumperLeftEvent = i2c_getEvent(rearBumperLeftAcquisition);

  createEvent(&bumperEvent2,"OR_REAR_REAR_LEFT");
  orEvents(rearBumperEvent, rearBumperLeftEvent, &bumperEvent2);

  /* rear right */
  rearBumperRightAcquisition = createPeriodicAcquisition(I2C_REAR_BUMPER_RIGHT_SLAVE, 
					    0 << I2C_REAR_BUMPER_RIGHT_BIT , 
					    1 << I2C_REAR_BUMPER_RIGHT_BIT, 50000);

  rearBumperRightEvent = i2c_getEvent(frontBumperRightAcquisition);

  createEvent(&bumperEvent3,"OR_REAR_ALL");
  orEvents(rearBumperRightEvent, &bumperEvent2, &bumperEvent3); 

  /* all event */
  createEvent(&bumperEvent,"OR_ALL_BUMPER");
  orEvents(&bumperEvent1, &bumperEvent3, &bumperEvent);
}

int GetBumperFromEvent(Event* origineevent)
{
   if (origineevent->signalingEvent == &bumperEvent0)
     return GetBumperFromEvent(&bumperEvent0);
   if (origineevent->signalingEvent == &bumperEvent1)
     return GetBumperFromEvent(&bumperEvent1);
   if (origineevent->signalingEvent == &bumperEvent2)
     return GetBumperFromEvent(&bumperEvent2);
   if (origineevent->signalingEvent == &bumperEvent3)
     return GetBumperFromEvent(&bumperEvent3);

   if (origineevent->signalingEvent == rearBumperRightEvent)
     return COLLI_REAR_R;
   if (origineevent->signalingEvent == rearBumperLeftEvent)
     return COLLI_REAR_L;
   if (origineevent->signalingEvent == rearBumperEvent)
     return COLLI_REAR;
   if (origineevent->signalingEvent == frontBumperRightEvent)
     return COLLI_FRONT_R;
   if (origineevent->signalingEvent == frontBumperLeftEvent)
     return COLLI_FRONT_L;
   if (origineevent->signalingEvent == frontBumperEvent)
     return COLLI_FRONT;
   return -1;
}

void DeleteBumperAcquisitions()
{
  deletePeriodicAcquisition(frontBumperAcquisition);
  deletePeriodicAcquisition(rearBumperAcquisition);
  deletePeriodicAcquisition(frontBumperRightAcquisition);
  deletePeriodicAcquisition(rearBumperRightAcquisition);
  deletePeriodicAcquisition(frontBumperLeftAcquisition);
  deletePeriodicAcquisition(rearBumperLeftAcquisition);
}
/******************************************************************************/
/*		FUNCTION TO MOVE																			*/
/******************************************************************************/
/* return true if the flag is dropped */
int Collision(int capteur)
{
 // todo rentre et sort le bras
  Bras* bras=GetNearestBras(currentTarget);
#ifdef DEBUG
  printf("Collision capteur :%d! \n",capteur);
#endif
  stopMove();
#ifdef LOG_DEBUG
  fptr1=fopen(LOG_FILENAME,"a");
  fprintf(fptr1,"Collision capteur :%d,  x:%lf y:%lf, t:%lf\n",capteur,getX(),getY(),getTheta());
  fclose(fptr1);
#endif
if(bras == NULL){
    EscapeCollision(capteur);
    return FALSE;
  }
  //  lcd_printf("ASTEROID DETECT\n");
  //sleep(5);

  /******** TODO test les bras ********/
  if(IsBrasOnPlanet(bras,currentTarget)){
    DropFlagWithBras(bras);
    RentreBras(bras);
    EscapeCollision(capteur);
    return TRUE;
  }
  else{
    EscapeCollision(capteur);
    return FALSE;
  }
}

void CollisionDetectedOnCaptor(int capteur)
{
  CollisionDetected(capteur,getX(),getY(),getTheta());
}

void EscapeCollision(int capteur)
{
    double SpeedR,SpeedL,L;
    int err;
    MovementDescriptor* evitement;
    Event endMvtEvent;
    Event stopEvent;
#ifdef DEBUG
      printf("Escape Collision Start\n");
#endif
      sleep(2);
#ifdef DEBUG
      printf("Escape Collision Star2t\n");
#endif
    stopMove();
    sleep(2);
#ifdef DEBUG
      printf("Escape Collision end stop move\n");
#endif
    CollisionDetectedOnCaptor(capteur);
#ifdef DEBUG
      printf("CollisionDetectedOnCaptor\n");
#endif
    err = CollisionGetSolution(&SpeedR,&SpeedL,&L);
    if(err!=OK){
#ifdef DEBUG
      printf("CollisionGetSolution error : %d\n",err);
#endif
      return;
    }
#ifdef DEBUG
      printf("CollisionGetSolution OK SR:%lf, SL: %lf\n",SpeedR,SpeedL);
#endif
    createEvent(&stopEvent,"STOP_EVENT");
    //CreateBumperEvents();
    //collisionAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_SLAVE, 1<< I2C_FRONT_BUMPER_BIT, 1<<I2C_FRONT_BUMPER_BIT,500000);
    //collisionEvent = i2c_getEvent(collisionAcquisition);
	 
    /************* TODO ***************/
    evitement = defineBasicLinearMovement(50,-GOTOLIB_SPEED_LINEAR_MVT);
    createEvent(&endMvtEvent,"END_EVITMEMENT_FINAL_EVENT");
    setEndMoveEvent(evitement, &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(evitement);
#ifdef DEBUG
      printf("Collision wait event\n");
#endif
    waitEvent(&stopEvent);
#ifdef DEBUG
    printf("Escape Collision FInished\n");
#endif
    DeleteBumperAcquisitions();
    //deletePeriodicAcquisition(collisionAcquisition);
    if (stopEvent.signalingEvent == &bumperEvent)
      {
	return EscapeCollision(GetBumperFromEvent(&bumperEvent));
      }
}

int GotoPlanetWithNb(int nbrplanet)
{
  return GotoPlanet(&Objectifs[nbrplanet]);
}

int GotoPlanet(Objectif * planet){
  MovementDescriptor* spline;
  Point directionDepart, directionFin, positionDepart;
  MovementDescriptor* rotation1;
  Event endMvtEvent;
  Event stopEvent;
  Bras * bras;
  int err=1;
  double angleDepart;
  double D,L;
  int nbrpoint=NBR_POINT;

  if(!initgotolib) init_gotoLibModule();

  currentTarget=planet;

  /* calcul de la trajectoire */

#ifdef DEBUG
  printf("Position de dpart : %lf %lf\n",getX(),getY());
  printf("objectif : %d\n",planet->PlaneteNb);
#endif
#ifdef LOG_DEBUG
  fptr1=fopen(LOG_FILENAME,"a");
  fprintf(fptr1,"GetTrajectoryToPlanet DEBUT x:%lf y:%lf, t:%lf\n",getX(),getY(),getTheta());
  fclose(fptr1);
#endif
  bras = GetFirstBras(planet);
  positionDepart.x = getX();
  positionDepart.y = getY();
  nbrpoint = NBR_POINT;
  err = GetTrajectoryToPlanet(positionDepart, planet , bras, TRUE , FALSE, splinePoints, &nbrpoint,&D,&L );
  directionDepart.x 	= 0;
  directionDepart.y 	= 10;
  directionFin.x 		= 10;
  directionFin.y 		= 10;

  /* execution de la trajectoire */
  if(err == OK){
    createEvent(&stopEvent,"STOP_EVENT");
    CreateBumperEvents();
    //collisionAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_SLAVE, 1<< I2C_FRONT_BUMPER_BIT, 1<<I2C_FRONT_BUMPER_BIT,500000);
    //collisionEvent = i2c_getEvent(collisionAcquisition);

    /* rotation pour s'aligner avec la spline*/
    // angle de la nouvelle direction
    if(positionDepart.x!=splinePoints[1].x){
      angleDepart = atan((splinePoints[1].y-positionDepart.y)/(splinePoints[1].x-positionDepart.x));
      if(getX()>splinePoints[1].x) angleDepart += M_PI;
    }
    else{
      if(getY()>splinePoints[1].y) angleDepart = -M_PI_2;
      else angleDepart = M_PI_2;
    }
    while(angleDepart>M_PI)  angleDepart -= 2*M_PI;
    while(angleDepart<-M_PI) angleDepart += 2*M_PI;
    
#ifdef DEBUG
    printf(" ready to start rotation, angle : %lf\n",angleDepart);
#endif
    rotation1 = defineRotationMovement(angleDepart);
    setMaxSpeed(rotation1,GOTOLIB_MAX_SPEED_ROTATION,GOTOLIB_MAX_SPEED_ROTATION,MAX_SPEED_PROPORTIONAL);
    createEvent(&endMvtEvent,"END_ROTATION_1_EVENT");
    setEndMoveEvent(rotation1, &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(rotation1);
    
    waitEvent(&stopEvent);
    if (stopEvent.signalingEvent == &bumperEvent){
      DeleteBumperAcquisitions();
      //deletePeriodicAcquisition(collisionAcquisition);
      return Collision(GetBumperFromEvent(&bumperEvent));
    }

      /* spline vers la planete */
#ifdef DEBUG
    printf("phase2 : spline mvt\n");
#endif
    spline = defineSplineMovement(directionDepart, directionFin, splinePoints, nbrpoint);
    setPIDParams(spline,0.5);
    setMaxSpeed(rotation1,GOTOLIB_MAX_SPEED_SPLINE,GOTOLIB_MAX_SPEED_SPLINE,MAX_SPEED_PROPORTIONAL);
    createEvent(&endMvtEvent,"END_SPLINE_CURVE");
    setEndMoveEvent(spline,  &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(spline);
      
    waitEvent(&stopEvent);
    if (stopEvent.signalingEvent == &bumperEvent){
      DeleteBumperAcquisitions();
      //deletePeriodicAcquisition(collisionAcquisition);
      return Collision(GetBumperFromEvent(&bumperEvent));
    }
#ifdef DEBUG
    printf("End Spline gotolib\n");
#endif
#ifdef LOG_DEBUG
    fptr1=fopen(LOG_FILENAME,"a");
    fprintf(fptr1,"GetTrajectoryToPlanet FIN x:%lf y:%lf, t:%lf\n",getX(),getY(),getTheta());
    fclose(fptr1);
#endif
    DeleteBumperAcquisitions();
    //deletePeriodicAcquisition(collisionAcquisition);
    return FinalApproachToPlanet(planet);
  }
  else {
#ifdef DEBUG
    printf("Trajectory Error : %d\n",err);
#endif
  }
  return FALSE;
}

int FinalApproach(int nbrplanet)
{
  return FinalApproachToPlanet(&Objectifs[nbrplanet]);
}

int FinalApproachToPlanet(Objectif * planet)
{
    MovementDescriptor* rotation1,*linear1,*linear2;
    Event endMvtEvent;
    Event stopEvent;
    double gt,gx,gy,D,L;
    int err,retry=0;
    Bras * bras = GetFirstBras(planet);
    if(bras==NULL) { // if planet == NULL=>bras=NULL
      return FALSE;
    }
#ifdef DEBUG
    printf ("Using flag : %d\n",bras->brasnbr);
    printf("Compute orientation finale\n");
#endif


    /* calcule orientation finale : */
    do{
    	gt = getTheta();
    	gx = getX();
    	gy = getY();
    	err = FinalApproch(gx, gy, bras, planet, &D, &L);
	if(err == ERROR_YOU_ARE_IN_THE_PLANET){

	}
    }while(err == ERROR_YOU_ARE_IN_THE_PLANET && retry++<MAX_OUT_OF_PLANET_RETRY);
    if(err != OK) {
#ifdef DEBUG
      printf("final approch error %d\n",err);
#endif
      return FALSE;
    }

    /* orientation*/
#ifdef DEBUG
    printf("Orientation finale : Direction : %lf Distance : %lf\n",D,L);
#endif
    /* sioux ! */
    while(D-gt>M_PI) D-=2*M_PI;
    while(D-gt<-M_PI) D-=2*M_PI;
    createEvent(&stopEvent,"STOP_EVENT");
    CreateBumperEvents();
    
    rotation1 = defineRotationMovement(D);
    setPIDParams(rotation1,0.4);  //0.1
    setEpsilon(rotation1, 0.01);
    setMaxSpeed(rotation1, GOTOLIB_MAX_SPEED_ROTATION, GOTOLIB_MAX_SPEED_ROTATION, MAX_SPEED_PROPORTIONAL);
    createEvent(&endMvtEvent,"END_ROTATION_FINALE_EVENT");
    setEndMoveEvent(rotation1, &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(rotation1);

    waitEvent(&stopEvent);
    if (stopEvent.signalingEvent == &bumperEvent)
      {
	DeleteBumperAcquisitions();
	//deletePeriodicAcquisition(collisionAcquisition);
	return Collision(GetBumperFromEvent(&bumperEvent));
      }

    /* perpare le bras */
    PrepareBras(bras);
    /* avance  */
#ifdef LOG_DEBUG
    fptr1=fopen(LOG_FILENAME,"a");
    fprintf(fptr1,"Oriantationfinale x:%lf y:%lf, t:%lf\n",getX(),getY(),getTheta());
    fclose(fptr1);
#endif
#ifdef DEBUG
    printf("Orientation finale : Direction : %lf Distance : %lf\n",D,L);
#endif
    linear1 = defineBasicLinearMovement(fabs(L),mysign(L)*GOTOLIB_SPEED_LINEAR_MVT);
    createEvent(&endMvtEvent,"END_LINEAR_FINAL_1_EVENT");
    setEndMoveEvent(linear1, &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(linear1);

    waitEvent(&stopEvent);
    if (stopEvent.signalingEvent == &bumperEvent)
      {
	DeleteBumperAcquisitions();
	//deletePeriodicAcquisition(collisionAcquisition);
	return Collision(GetBumperFromEvent(&bumperEvent));
      }
#ifdef DEBUG
    printf("Orientation finale : FIN\n");
#endif

    /* Lache le drapeau !!!!!!!!! */
    DropFlagWithBras(bras);

    /* Rentre le bras */
    RentreBras(bras);
    
    /* Recule */
    linear2 = defineBasicLinearMovement(fabs(L),-mysign(L)*GOTOLIB_SPEED_LINEAR_MVT);
    createEvent(&endMvtEvent,"END_LINEAR_FINAL_2_EVENT");
    setEndMoveEvent(linear2, &endMvtEvent);
    orEvents(&bumperEvent, &endMvtEvent, &stopEvent);
    startMove(linear2);

    waitEvent(&stopEvent);
    if (stopEvent.signalingEvent == &bumperEvent)
      {
	DeleteBumperAcquisitions();
	//deletePeriodicAcquisition(collisionAcquisition);
	return Collision(GetBumperFromEvent(&bumperEvent));
      }
    DeleteBumperAcquisitions();
    //deletePeriodicAcquisition(collisionAcquisition);
    
#ifdef DEBUG
    printf("Trajectory Fin\n");
    sleep(1);
#endif
#ifdef LOG_DEBUG
    fptr1=fopen(LOG_FILENAME,"a");
    fprintf(fptr1,"Fin x:%lf y:%lf, t:%lf\n",getX(),getY(),getTheta());
    fclose(fptr1);
#endif
    return TRUE;
}


