#include <stdio.h>
#include <errno.h>
#include <unistd.h>
#include <stdlib.h>

#include "periodServer.h"
#include <i2cManager.h>

#define DEBUG_MODULE_NAME PERSERVER
#include "debug.h"

/***************** Variables globales  *******************/

int NB_SLAVES = NB_I2C_SLAVES + 1;
slaveStruct* slaveList;
pthread_mutex_t slaveList_mutex;

/************ Pour grer la liste des Esclaves  **********/
void init_slaveList()
{
  int i;
 
  slaveList = (slaveStruct *) malloc (NB_SLAVES * sizeof (slaveStruct));
  pthread_mutex_init(&slaveList_mutex,NULL);

  for(i=0;i<NB_SLAVES;++i)
    {
      slaveList[i].slave_addr = 0x00;
      slaveList[i].periodMin=0;
    }
}

slaveStruct * find_slave (unsigned char slave_addr)
{
  int i=0;

   for(i=0;i<NB_SLAVES;++i)	
    if (slaveList[i].slave_addr == slave_addr)
	  return &slaveList[i];
 
  return NULL;
}

slaveStruct * add_slave (unsigned char slave_addr)
{
  int i=0;

  for(i=0;i<NB_SLAVES;++i)	
      if (slaveList[i].slave_addr == slave_addr)
		  return &slaveList[i];
      
  i=0;
  while(slaveList[i].slave_addr!=0x00 && i < NB_SLAVES)
     ++i;

  pthread_mutex_lock(&slaveList_mutex); 
    
  slaveList[i].slave_addr=slave_addr;
  pthread_mutex_init(&(slaveList[i].slave_mutex),NULL);
  
  pthread_mutex_unlock(&slaveList_mutex);
  return &slaveList[i];
}

/************ Thread d'acquisition d'un esclave **********/
void * slaveThread (void * arg)
{
  int result;
  slaveStruct * slavePointer = (slaveStruct *) arg;
  periodicTask* tempTaskPointer;

  while (1)
    {
      result = i2cGet(slavePointer->slave_addr,0xFF);
      debug_printf(DEBUG_MODULE_NAME," Getting i2x .... \n");

      pthread_mutex_lock(&(slavePointer->slave_mutex));  
     
      tempTaskPointer = slavePointer->taskList;
      while (tempTaskPointer!=NULL)
		{
		  debug_printf(DEBUG_MODULE_NAME,"Testing result=%d mask=%d value=%d .... \n",result,tempTaskPointer->mask, tempTaskPointer->waitedValue);
		  if ((result & tempTaskPointer->mask) == tempTaskPointer->waitedValue)
			{	 
			  broadcastEvent (&(tempTaskPointer->endEvent));
			  debug_printf(DEBUG_MODULE_NAME,"broadcasting ... %d\n",tempTaskPointer->Period);
			}
		  tempTaskPointer=tempTaskPointer->next; 
		}
      pthread_mutex_unlock(&(slavePointer->slave_mutex));  

	  debug_printf(DEBUG_MODULE_NAME,"Sleeping : %d\n", slavePointer->periodMin);  
      usleep(slavePointer->periodMin);
    }
}

void start_slaveAcquisition(slaveStruct* slavePointer)
{
  pthread_create(&(slavePointer->slaveThreadId),NULL,slaveThread,(void * )slavePointer);
}
 
/****** Gestion de la liste des taches periodiques *******/
void push_periodicTask (slaveStruct* slavePointer,  periodicTask * Task)
{
  periodicTask * tempNext;

  pthread_mutex_lock(&(slavePointer->slave_mutex)); 

  if (slavePointer->taskList!=NULL)
	tempNext = slavePointer->taskList;
  else
	tempNext = NULL;

  slavePointer->taskList = Task;
  Task->next=tempNext;

  pthread_mutex_unlock(&(slavePointer->slave_mutex));  
}

periodicTask* add_periodicTask (unsigned char slave, unsigned char value, unsigned char mask, int Period)
{
  slaveStruct * slavePointer;
  periodicTask* cmd;

  cmd = (periodicTask*)malloc(sizeof(periodicTask));

  cmd->waitedValue=value;
  cmd->mask=mask;
  cmd->slave = slave;

  cmd->next=NULL;
  cmd->Period = Period;
  
  createEvent (&(cmd->endEvent),""); 

  slavePointer = add_slave(slave);

  push_periodicTask(slavePointer,cmd);
  
  if (Period < (slavePointer->periodMin) || !slavePointer->taskList->next)
    {
	  slavePointer->periodMin = Period;
    }

  if (slavePointer->taskList->next==NULL)
      {	start_slaveAcquisition(slavePointer);
        debug_printf(DEBUG_MODULE_NAME,"Starting Thread  : 0x%x \n ", slavePointer->slave_addr);  
      }
  return cmd;
}

void remove_periodicTask (periodicTask* task2delete)
{
   periodicTask* taskTemp;
   slaveStruct * slavePointer = find_slave(task2delete->slave);
  
   pthread_mutex_lock (&(slavePointer->slave_mutex));

   if (slavePointer->taskList== task2delete)
	{
	  slavePointer->taskList=task2delete->next;
	  if (slavePointer->taskList==NULL)
      	    {   int i;
		
                free(task2delete);
	        i = pthread_cancel(slavePointer->slaveThreadId);
	        debug_printf(DEBUG_MODULE_NAME,"Cancel_Thread : %d\n", i);  
	        pthread_mutex_unlock(&(slavePointer->slave_mutex));  
		return;
	    }
	}
   else
	{ 
	  taskTemp=slavePointer->taskList;
	  while (taskTemp->next != NULL && taskTemp->next!=task2delete)
		taskTemp=taskTemp->next;
	  if (!taskTemp->next)
		{
		  pthread_mutex_unlock(&(slavePointer->slave_mutex));  
		  return;
		}
	  taskTemp->next = task2delete->next;
	}

   if(slavePointer->periodMin==task2delete->Period)
	{
	  slavePointer->periodMin = slavePointer->taskList->Period;
	  taskTemp=slavePointer->taskList;
	  while (taskTemp)
		{
		  if(taskTemp->Period < slavePointer->periodMin)
			slavePointer->periodMin = taskTemp->Period;
		  taskTemp=taskTemp->next;
		}
	}

   
   free(task2delete);

   pthread_mutex_unlock(&(slavePointer->slave_mutex));
}

/************ Pour acceder de l'exterieur ****************/

periodicTask *  createPeriodicAcquisition(unsigned char slave, unsigned char value, unsigned char mask, int Period)
{ 
  return ( add_periodicTask ( slave, value, mask, Period));
} 

void deletePeriodicAcquisition(periodicTask * task2delete)
{
  remove_periodicTask (task2delete);
}

Event * i2c_getEvent (periodicTask * task2wait)
{
  return(&task2wait->endEvent);
}

/***************** Lancement et Arret *******************/

void perServer_initModule()
{
  debug_printf(DEBUG_MODULE_NAME,"Initializing periodic Server\n");
  init_slaveList();
}

void perServer_cleanUpModule()
{
  debug_printf(DEBUG_MODULE_NAME,"Cleaning periodic Server\n");
}













