#ifndef __MOVE_H__
#define __MOVE_H__

#include "pthread.h"
#include "robot_geometry.h"
#include "spline.h"
#include "events.h"

typedef struct WatchPoint
{
  unsigned char (* testFunction) (double, double, double, void*);
  void* params;
  unsigned char permanent;

  Event* event;

/*  pthread_cond_t event;
  pthread_mutex_t mutex;*/
  struct WatchPoint* next;
} WatchPoint;

#define MAX_SPEED_PROPORTIONAL 1
#define MAX_SPEED_NOT_PROPORTIONAL 0

#define INTEGRAL_MEMORY_SIZE 20

typedef struct MovementDescriptor
{
  unsigned char moveType;
  double* coords;
  int nbCoords;
  double epsilon;
  WatchPoint* watchPointList;
  Event* endOfMove;
  double maxSpeedR;
  double maxSpeedL;
  int isMaxProportional;
  double k;
  double ki;
  double a_ref;
  double b_ref;
  double errorMemory[INTEGRAL_MEMORY_SIZE];
  int errorMemoryIndex;
} MovementDescriptor;

extern int move_initModule();

//extern void defineLinearMovement(double length);

extern MovementDescriptor* defineRotationMovement(double deltaTheta);
extern MovementDescriptor* defineLinearMovement(double length);
extern MovementDescriptor* defineXYMovement(double x, double y);
extern MovementDescriptor* defineSplineMovement(Point directionDepart, Point directionFin, Point *points, int nbPoints);
extern MovementDescriptor* defineBasicLinearMovement(double length, double linearSpeed);

extern void setMaxSpeed(MovementDescriptor* move, double maxSpeedR, double maxSpeedL, int isProportional);
extern void setPIDParams(MovementDescriptor* move, double k);
extern void setRefPoint(MovementDescriptor* move, double x, double y);
extern void setEpsilon(MovementDescriptor* move, double epsilon);

extern void setTargetDist(MovementDescriptor* move, double targetDist);
extern void setFinalDist(MovementDescriptor* move, double finalDist);
extern void setFinalK(MovementDescriptor* move, double finalK);


extern void setEndMoveEvent(MovementDescriptor* move,  Event* event);

extern void startMove(MovementDescriptor* move);
extern void stopMove();

extern void addWatchPoint(MovementDescriptor* move,
						  Event* event,
					    unsigned char (* testFunction) (double, double, double,void*),
					    void* params);

//extern WatchPoint* addWatchPoint(MovementDescriptor* move,
//				 unsigned char (* testFunction) (double, double, double,void*),
//				 void* params);

extern void waitWatchPoint(WatchPoint* watchPoint);

extern void setPermanentWatchPoint(WatchPoint* watchPoint);

extern void deleteWatchPoint(MovementDescriptor* move, WatchPoint* watchPoint);

extern void setRobotPos(double newX, double newY, double newTheta);
extern void getRobotPos(double *X, double *Y, double *T);
//extern void goLinearMovement();

//extern void defineLinearMovementEvent(double length);

//extern void waitLinearMovementEvent();

double getX();
double getY();
double getTheta();

extern void resetStop();

extern void setSpeed(double linearSpeed, double angularSpeed);

extern void setPWMLimitation(int limit);
extern void enablePWMLimitation();
extern void disablePWMLimitation();


extern void MoveLinearBasic(double length,int linearSpeed);
#endif // __MOVE_H__
