#ifndef __ROBOT_GEOMETRY_H__
#define __ROBOT_GEOMETRY_H__

#include <math.h>
#include "point.h"

#define RAYON_ROBOT 	200. //mm
#define K (1/239.237)        // conversion codeur/mm : 1 pas codeur = K mm
#define e 340.0/2              // demie entre deux roues

/* modes d'approche des planetes */
#define APPROCH_NORMAL  0
#define APPROCH_ANGLE   1
#define APPROCH_NEAREST 2   // par defaut

#define TRUE  1
#define FALSE 0 

#define NBR_OBJECTIFS          7
#define BRAS_NUMBER            7
#define NBR_COLLISION_DETECTOR 10

typedef struct Bras_
{
   int brasnbr;
   double r;
   double theta;
   int flagAvailable;
   double rMinOnPlanet;
} Bras;


typedef struct Objectif_
{
  Point Coordonnees;
  int PlaneteNb;
  int Priorite;
  int Etat;
  int rayon;
  int approchemode;
  int thetaapproche;
  int Drapeaux [BRAS_NUMBER];
} Objectif;

extern Objectif * Objectifs;
extern Bras * BrasList;

/************ Coordonnees des planetes ************/
#define  PLANET_STAR         0
#define  PLANET_MY_RIGHT     2
#define  PLANET_MY_LEFT      1
#define  PLANET_ENNEMY_RIGHT 4
#define  PLANET_ENNEMY_LEFT  3
#define  PLANET_VELCRO_RIGHT 6
#define  PLANET_VELCRO_LEFT  5
#define  X_Coord {1500,660,660,2340,2340,1500,1500}
#define  Y_Coord {1050,660,1440,660,1440,300,1800}
#define  Obj_rayon {160,120,120,120,120,120,120}
#define  DefaultPriorite {9,7,7,5,5,3,3};
/************* Parametres des bras&drapeaux ***************/
#define FLAG_STAR         0
#define FLAG_MY_RIGHT     2
#define FLAG_MY_LEFT      1
#define FLAG_ENNEMY_RIGHT 4
#define FLAG_ENNEMY_LEFT  3
#define FLAG_VELCRO_RIGHT 6
#define FLAG_VELCRO_LEFT  5
#define Bras_r  {300,300,300,150,150,150,150} /* distance des bras au centre du robot en mm*/
#define Bras_theta  {0,-5*M_PI/6,5*M_PI/6,-M_PI/4.,M_PI/4,-M_PI/4.,M_PI/4} /* direction des bras dans le repere indirect */
#define Bras_flagAvailable {TRUE,TRUE,TRUE,TRUE,TRUE,FALSE,FALSE} /* oui sauf velcro */
#define Bras_rMinOnPlanet {30,30,30,30,30,30,30} /* 30 mm*/

/************ Parametres des capteurs de choc ***********/
#define COLLI_FRONT 	0
#define COLLI_FRONT_L 	1
#define COLLI_FRONT_R 	2
#define COLLI_REAR 	3
#define COLLI_REAR_L 	4
#define COLLI_REAR_R 	5
#define COLLI_RIGHT_F 	6
#define COLLI_RIGHT_R 	7
#define COLLI_LEFT_F 	8
#define COLLI_LEFT_R 	9

#define Colli_detector_L {200,200,200,200,200,200,200,200,200,200} /* position des capteurs */
#define Colli_detector_Theta {0,0,0,0,0,0,0,0,0,0} /*position des capteurs */
#define Colli_detector_DirDetect {0,0,0,0,0,0,0,0,0,0} /* direction de dtection (dans le repere indirect)*/

#define RAYON_OBSTACLE 70.

/******* pour le soft d'approche finale dans les trajectoires *******/
/* forme de la croix qu'il y a sous le robot */
/* ICI qu'on prend de la marge */
#define DXf  	60.   /* Demi Largeur de la barre longitudinale avant */
#define DXr  	30.   /* Demi Largeur de la barre longitudinale arriere	 */
#define DYf 	60.   /* Demi Largeur de la barre transversale avant */
#define DYr	60.   /* Demi Largeur de la barre transversale arriere */


#endif //__ROBOT_GEOMETRY_H__
