#include "spline.h"

void spline (SplineCurve curve, double t, Point* result)
     /* donne un Catmull-Rom-Spline entre [0..curve.nbPoints-1] */
{
  Point m0; //derivee au point 0 de notre sous-part de la courbe
  Point m1; //derivee au point 1 de notre sous-part de la courbe
  Point tmp;
  int i = t; // sous-part de la courbe
  t -= i; // l'endroit dans notre sous-part

  if (i == 0)
    {
      tmp.x = curve.directionDepart.x;
      tmp.y = curve.directionDepart.y;
    }
  else
    {
      tmp.x = curve.p[i].x - curve.p[i-1].x;
      tmp.y = curve.p[i].y - curve.p[i-1].y;
    }
  m0.x = (1-smoothness)*(1+decalage)/2.0*tmp.x + (1-smoothness)*(1-decalage)/2.0*(curve.p[i+1].x - curve.p[i].x);
  m0.y = (1-smoothness)*(1+decalage)/2.0*tmp.y + (1-smoothness)*(1-decalage)/2.0*(curve.p[i+1].y - curve.p[i].y);

  if (i == (curve.nbPoints-2))
    {
      tmp.x = curve.directionFin.x;
      tmp.y = curve.directionFin.y;
    }
  else
    {
      tmp.x = curve.p[i+2].x - curve.p[i+1].x;
      tmp.y = curve.p[i+2].y - curve.p[i+1].y;
    }
  m1.x = (1-smoothness)*(1+decalage)/2.0*(curve.p[i+1].x - curve.p[i].x) + (1-smoothness)*(1-decalage)/2.0*tmp.x;
  m1.y = (1-smoothness)*(1+decalage)/2.0*(curve.p[i+1].y - curve.p[i].y) + (1-smoothness)*(1-decalage)/2.0*tmp.y;

  result->x = (2*t*t*t - 3*t*t + 1)*curve.p[i].x + (t*t*t - 2*t*t + t)*m0.x
    + (t*t*t - t*t)*m1.x + ((-2)*t*t*t + 3*t*t)*curve.p[i+1].x;
  result->y = (2*t*t*t - 3*t*t + 1)*curve.p[i].y + (t*t*t - 2*t*t + t)*m0.y
    + (t*t*t - t*t)*m1.y + ((-2)*t*t*t + 3*t*t)*curve.p[i+1].y;
}

double getNextPoint(SplineCurve curve, Point pos, double tStart)
{
  double currentT = tStart;
  Point p;
  double length = 0;
  int maxT = curve.nbPoints - 1;
  
  spline(curve, currentT, &p);


  length=(p.x - pos.x)*(p.x - pos.x) + (p.y - pos.y)*(p.y - pos.y);
  
  while ((length < minLength) && (currentT < maxT))
    {    
      currentT+=deltaT;  
      spline(curve, currentT, &p);
      length=(p.x - pos.x)*(p.x - pos.x) + (p.y - pos.y)*(p.y - pos.y);
    }

  if (currentT > maxT) currentT = maxT;

  return currentT;
}
