#include "robot.h"
#include "periodServer.h"

#include <unistd.h>

#define DEBUG_MODULE_NAME MAIN
#include "debug.h"

#include "move.h"

#define PI 3.14159

int main()
{
  periodicTask* jackOn;
  periodicTask* jackOff;
  Event* jackOffEvent;
  Event* jackOnEvent;

  periodicTask* frontBumperAcquisition;
  Event* frontBumperEvent;

  MovementDescriptor* moveToBigStar;
  Event endMoveToBigStarEvent;

  MovementDescriptor* moveBack;
  Event endMoveBackEvent;

  MovementDescriptor* rotation1;
  Event endRotation1Event;

  MovementDescriptor* rotation2;
  Event endRotation2Event;

  MovementDescriptor* rotation3;
  Event endRotation3Event;

  MovementDescriptor* rotation4;
  Event endRotation4Event;

  Point *splinePoints;
  Point directionDepart;
  Point directionFin;
  MovementDescriptor* spline;
  
  Event pwmLimitationEvent;

  Event stopEvent;
  Event stopEvent2;
  
  int launchBigStarPMIOK = 1; // we are optimistic

  debug_printf(DEBUG_MODULE_NAME,"Start soft\n");

  shared_initModule();

  i2c_initModule();

  INIThctl();

  move_initModule();
  
  electro_initModule();

  relay_initModule();

  perServer_initModule();

  debug_printf(DEBUG_MODULE_NAME,"BEGIN of SCENARIO\n");


  globalState->ServosCons[0]=0;

  // Gestion START JACK
  //******************
  jackOn  = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 1 << I2C_STARTJACK_BIT, 1 << I2C_STARTJACK_BIT, 500000);
  jackOff = createPeriodicAcquisition(I2C_STARTJACK_SLAVE, 0                     , 1 << I2C_STARTJACK_BIT, 500000);
  jackOffEvent = i2c_getEvent(jackOff);
  jackOnEvent = i2c_getEvent(jackOn);
  
  //waitEvent(jackOffEvent);
  //waitEvent(jackOnEvent);
  //waitEvent(jackOffEvent);

  deletePeriodicAcquisition(jackOn);
  deletePeriodicAcquisition(jackOff);

  // Avancement jusqu'a la planete centrale   
  //***************************************
  moveToBigStar = defineXYMovement(1330.0,0.0);
  setPIDParams(moveToBigStar,0.8); //1.0  // 1.1
  setEpsilon(moveToBigStar,3.0);
      //setRefPoint(moveToBigStar,1.0,0.0);
  createEvent(&endMoveToBigStarEvent,"END_MOVE_TO_BIG_STAR_EVENT");
  setEndMoveEvent(moveToBigStar,  &endMoveToBigStarEvent);
  // bumper management
  frontBumperAcquisition = createPeriodicAcquisition(I2C_FRONT_BUMPER_SLAVE, 1 << I2C_FRONT_BUMPER_BIT, 1 << I2C_FRONT_BUMPER_BIT, 5000);
  frontBumperEvent = i2c_getEvent(frontBumperAcquisition);

  createEvent(&stopEvent,"STOP_EVENT");
  orEvents(frontBumperEvent, &endMoveToBigStarEvent, &stopEvent);

  debug_printf(DEBUG_MODULE_NAME,"Start move to big star\n");
  startMove(moveToBigStar);

  armMotor(ON);
  globalState->ServosCons[0]=240;

  //  waitEvent(&endMoveToBigStarEvent);

  waitEvent(&stopEvent);
  deletePeriodicAcquisition(frontBumperAcquisition);
  debug_printf(DEBUG_MODULE_NAME,"0x%x - 0x%x\n",stopEvent.signalingEvent,frontBumperEvent);
  if (stopEvent.signalingEvent == frontBumperEvent)
    {
   //   if (getX() < 850.0)
	{
	  //	  resetStop();
	  // reduce the max speed
	  setMaxSpeed(moveToBigStar, 20, 20, MAX_SPEED_PROPORTIONAL);
	  setFinalK(moveToBigStar, 0.5);

	  usleep(500000);

	  createEvent(&pwmLimitationEvent,"PWM LIMITATION");
	  createEvent(&stopEvent2,"STOP_EVENT_2");
	  orEvents(&pwmLimitationEvent, &endMoveToBigStarEvent, &stopEvent2);

	  setPWMLimitationEvent(&pwmLimitationEvent); 
	  setPWMLimitation(50);
	  enablePWMLimitation();
	  
	  waitEvent(&stopEvent2);

	  if (stopEvent2.signalingEvent == &pwmLimitationEvent)
	    {
	      resetStop();
	      launchBigStarPMIOK=0;
	    }

	}
      //stopMove();
    }

  printf("%lf - %lf - %lf\n", getX(),getY(),getTheta());

  debug_printf(DEBUG_MODULE_NAME,"End move to big star\n");
  
  if (launchBigStarPMIOK==1)
    {
      sleep(1);
      
      electroOn(1);
      
      sleep(1);
    }

  armMotor(OFF);
  
      
  globalState->ServosCons[0]=0;


  // Recul jusqu'au point au centre des 2 planetes "chez nous"
  //**********************************************************
  moveBack = defineXYMovement(850.0,0.0);
  setRefPoint(moveBack,-5.0,0.0); //-1.0
  setPIDParams(moveBack,0.5);  //0.1
  createEvent(&endMoveBackEvent,"END_MOVE_BACK_EVENT");
  setEndMoveEvent(moveBack,  &endMoveBackEvent);

  debug_printf(DEBUG_MODULE_NAME,"Start move back\n");
  startMove(moveBack);
  waitEvent(&endMoveBackEvent);

  globalState->ServosCons[1]=127;
  armMotor(OFF);
  
  // Rotation et depose du premier drapeau
  //**************************************
  rotation1 = defineRotationMovement(PI/2);
  setMaxSpeed(rotation1,0,127.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation1Event,"END_ROTATION_1_EVENT");
  setEndMoveEvent(rotation1,  &endRotation1Event);
  startMove(rotation1);
  waitEvent(&endRotation1Event);
  sleep(3);
  electroOn(2);
  sleep(1);
  globalState->ServosCons[1]=0;

  // Retour au centre
  //******************
  rotation2 = defineRotationMovement(0.0);
  setMaxSpeed(rotation2,0,127.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation2Event,"END_ROTATION_2_EVENT");
  setEndMoveEvent(rotation2,  &endRotation2Event);
  startMove(rotation2);
  waitEvent(&endRotation2Event);

  // Rotation et depose du second drapeau
  //*************************************
  rotation3 = defineRotationMovement(-PI/2);
  setMaxSpeed(rotation3,127.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation3Event,"END_ROTATION_3_EVENT");
  setEndMoveEvent(rotation3,  &endRotation3Event);
  startMove(rotation3);  
  waitEvent(&endRotation3Event);
  sleep(3);
  electroOn(2);
  sleep(1);

  // Retour au centre
  //******************
  rotation4 = defineRotationMovement(0.0);
  setMaxSpeed(rotation4,127.0,0.0,MAX_SPEED_NOT_PROPORTIONAL);
  createEvent(&endRotation4Event,"END_ROTATION_4_EVENT");
  setEndMoveEvent(rotation4,  &endRotation4Event);
  startMove(rotation4);  
  waitEvent(&endRotation4Event);

  // Mouvement jusqu'a une des planetes avec Velcro
  //***********************************************
  splinePoints = (Point*) malloc(3*sizeof(Point));
  splinePoints[0].x = getX();
  splinePoints[0].y = getY();
  splinePoints[1].x = 1200.0;
  splinePoints[1].y = 200.0;
  splinePoints[2].x = 1600.0;
  splinePoints[2].y = 600.0;
  directionDepart.x = 0;
  directionDepart.y = 10;
  directionFin.x = 10;
  directionFin.y = 10;
  
  spline = defineSplineMovement(directionDepart, directionFin, splinePoints, 3);
  setPIDParams(spline,0.3);
  startMove(spline);

  sleep(30);

  debug_printf(DEBUG_MODULE_NAME,"END OF SCENARIO...\n");

  return 0;
}




