#include <stdio.h>
#include <pthread.h>
#include <stdlib.h>


#include "Intel.h"
#include "trajectory.h"
#include "robot_geometry.h"
#include "gotolib.h"

#define DEBUG_MODULE_NAME INTEL
#include "debug.h"


/************************** variables globales ***********************
#define  NBR_OBJECTIFS 7
#define  X_Coord {1500,660,660,2340,2340,1500,1500}
#define  Y_Coord {1050,660,1440,660,1440,300,1800}
#define  Obj_rayon {160,120,120,120,120,120,120}
#define  DefaultPriorite{9,7,7,5,5,3,3};

/************* Parametres des bras **************

#define Bras_r  {300,300,300,150,150,150,150}
#define Bras_theta  {0,-5*M_PI/6,5*M_PI/6,-M_PI/4,M_PI/4,-M_PI/4,M_PI/4}
#define Bras_flagAvailable {TRUE,TRUE,TRUE,TRUE,TRUE,TRUE,TRUE}
#define Bras_rMinOnPlanet {30,30,30,30,30,30,30}
*/
#define CONF_FILE  "Intel.conf"

Objectif* Objectifs;
Bras* BrasList;



/************************** fonction d'utilite generale ************************/

void obj_copy( Objectif * objDest, Objectif * objSrc )
{
  objDest->Coordonnees.x = objSrc->Coordonnees.x;
  objDest->Coordonnees.y = objSrc->Coordonnees.y;
  objDest->PlaneteNb = objSrc->PlaneteNb;
  objDest->Priorite = objSrc->Priorite;
  objDest->Etat = objSrc->Etat;
  objDest->rayon = objSrc->rayon;
  // Copier l'Event
}


Objectif * search_obj (int planetenb)
{
  int i;
  for (i=0;i<7;++i)
   if(Objectifs[i].PlaneteNb==planetenb) return &Objectifs[i]; 

  return 0;
}

/************************** Pour le tri ************************/

void intel_organize()
{
  int i,j=0;
  Objectif objTemp;
  
  for (i=6;i>-1;--i)
   {
        j=i;	
        while (j<6 && Objectifs[j].Priorite < Objectifs[j+1].Priorite )
          {
            obj_copy (&objTemp,&Objectifs[j+1]);
	    obj_copy (&Objectifs[j+1],&Objectifs[j]);
            obj_copy (&Objectifs[j],&objTemp);
            ++j;
          }
   }
}

/************************** Pour le chargement des objectifs et des bras  ************************/
void load_bras()
{
  int i =0;
  double sBras[BRAS_NUMBER]=Bras_r;
  double sTheta[BRAS_NUMBER]=Bras_theta;
  int sflagA[BRAS_NUMBER]=Bras_flagAvailable;
  double rmin[BRAS_NUMBER]=Bras_rMinOnPlanet;

  for (i=0;i<BRAS_NUMBER;++i)
    {
      BrasList[i].brasnbr=i;
      BrasList[i].r= sBras[i];
      BrasList[i].theta = sTheta[i];	
      BrasList[i].flagAvailable = sflagA[i];
      BrasList[i].rMinOnPlanet= rmin[i];
    }
}

void load_Objectifs(FILE * fd)
{
  char fcar[2];
  char string[200];
  int objNb;
  int i=0;
  int planetx[NBR_OBJECTIFS]=X_Coord;
  int planety[NBR_OBJECTIFS]=Y_Coord;
  double planetrayon[NBR_OBJECTIFS]=Obj_rayon;

  while (fscanf(fd,"%c",fcar)!=EOF)
   {
     if (fcar[0] == '#')
      {  
        // Exploit : La securite voudrait qu'on le fasse lettre par lettre
        fscanf (fd,"%[^\n]",string);	
        //printf ("-- %s \n",string); 
        fscanf (fd,"%c",fcar);	
      }
     else
       if(fcar[0]!= '\n')
        {
	  fscanf(fd,"%[^1-9]",string);
	  fscanf(fd,"%c",fcar);
          fcar[1]=0;
          objNb = atoi(fcar)-1;
          //printf(" -- objNb :  %d \n", objNb);	  
          Objectifs[objNb].Coordonnees.x = planetx[objNb]; 
          Objectifs[objNb].Coordonnees.y = planety[objNb];
          Objectifs[objNb].PlaneteNb = objNb + 1;
	  
	  fscanf(fd,"%[^1-9]",string);
	  fscanf(fd,"%c",fcar);
          fcar[1]=0;
          //printf(" -- priorite :  %c \n", fcar[0]);	  
 
          Objectifs[objNb].Priorite = atoi(fcar);
          Objectifs[objNb].Etat = TO_DO;
	  Objectifs[objNb].rayon = planetrayon[objNb]; 
          fscanf (fd,"%c",fcar);	
	  
	  for (i=1;i<BRAS_NUMBER;++i)
	    Objectifs[objNb].Drapeaux[i]= -1;
	  Objectifs[objNb].Drapeaux[0]=objNb;
        }
   }	  
  
  intel_organize();
}

void load_Default_Objectifs ()
{
  int i,j;
  int planetx[NBR_OBJECTIFS]=X_Coord;
  int planety[NBR_OBJECTIFS]=Y_Coord;
  int planetrayon[NBR_OBJECTIFS]=Obj_rayon; 
  int planetpriority[NBR_OBJECTIFS]=DefaultPriorite;
  for (i=0;i<NBR_OBJECTIFS;++i)
    {
      Objectifs[i].Coordonnees.x = planetx[i]; 
      Objectifs[i].Coordonnees.y = planety[i];
      Objectifs[i].Priorite = planetpriority[i];
      Objectifs[i].Etat = TO_DO;
      Objectifs[i].rayon = planetrayon[i]; 
      Objectifs[i].PlaneteNb = i; 
      for (j=1;j<BRAS_NUMBER;++j)
	Objectifs[i].Drapeaux[j]= -1;
       Objectifs[i].Drapeaux[0]=i;
    }
}


/************************** Coeur du programme  ************************/

void intel_activate ()
{
  int i,end=0;
  int j;
  int tryNb =0;

  while(!end) 
   { 
     end = 1;
     for(i=0;i<7;++i)  
        if (Objectifs[i].Etat!=CANCEL)
	  {
	    if (i<6)
	      tryNb = Objectifs[i].Priorite - Objectifs[i+1].Priorite;
	    else 
	      tryNb = Objectifs[i].Priorite;
	    end = 0;

	    if (GotoPlanet(&Objectifs[i]))
	      Objectifs[i].Etat = CANCEL;
	    else 
	      for (j=0;j<tryNb;++j)
		GotoPlanet(&Objectifs[i]);
	  }
  }

  debug_printf(DEBUG_MODULE_NAME,"No objective remaining \n");
}

/************************** Initialisation du module  ************************/

void intel_init_module ()
{
   FILE * fd = fopen (CONF_FILE,"r");
   Objectifs = (Objectif *) malloc (NBR_OBJECTIFS * sizeof(Objectif));
   BrasList = (Bras *) malloc ( BRAS_NUMBER* sizeof(Bras));

   load_Default_Objectifs();   

   if (fd)
       load_Objectifs(fd);
   load_bras();
}
